
from datetime import datetime

from turbogears.i18n.format import *

dt = datetime(2005, 2, 5) # Saturday, 5rd Feb 2005

def test_get_countries():
    print get_countries()
    print get_countries("eo")
    print get_countries("en_US")

def test_get_languages():
    print get_languages()
    print get_languages("eo")
    print get_countries("en_US")

def test_get_month_names():
    print get_month_names()
    print get_abbr_month_names()

    print get_month_names("eo")
    print get_abbr_month_names("eo")

    print get_month_names("en_US")
    print get_abbr_month_names("en_US")

def test_get_weekday_names():
    print get_weekday_names()
    print get_abbr_weekday_names()

    print get_weekday_names("eo")
    print get_abbr_weekday_names("eo")

    print get_weekday_names("en_US")
    print get_abbr_weekday_names("en_US")

def test_format_numbers():
    assert format_decimal(4.9, 0, "en") == "5"
    assert format_decimal(4.9, 1, "en") == "4.9"
    assert format_decimal(5.0, 1, "en") == "5.0"
    assert format_decimal(5.1, 1, "de") == "5,1"

    assert format_number(50, "en_US") == "50"
    assert format_number(50, "de") == "50"

    assert format_number(500, "en_US") == "500"
    assert format_number(500, "de") == "500"

    assert format_number(5000, "en_US") == "5,000"
    assert format_number(5000, "de") == "5.000"

    assert format_number(50000, "en_US") == "50,000"
    assert format_number(50000, "de") == "50.000"

    assert format_number(500000, "en_US") == "500,000"
    assert format_number(500000, "de") == "500.000"

    assert format_number(5000000, "en_US") == "5,000,000"
    assert format_number(5000000, "de") == "5.000.000"

    assert format_decimal(5000000.78781, 5, "en_US") == "5,000,000.78781"
    assert format_decimal(5000000.78781, 5, "de") == "5.000.000,78781"

    assert format_currency(5000000.78781, "en_US") == "5,000,000.79"
    assert format_currency(5000000.78781, "de") == "5.000.000,79"

def test_format_date():
    assert format_date(dt, "long", "eo") == u"2005-februaro-05"
    assert format_date(dt, "medium", "eo") == u"2005-feb-05"
    assert format_date(dt, "short", "eo") == u"05-02-05"
    assert format_date(dt, "full", "eo") == u"sabato, 05-a de februaro 2005"

    assert format_date(dt, "long", "de") == u"05. Februar 2005"
    assert format_date(dt, "medium", "de") == u"05.02.2005"
    assert format_date(dt, "short", "de") == u"05.02.05"
    assert format_date(dt, "full", "de") == u"Samstag, 05. Februar 2005"

    assert format_date(dt, "short", "en",
            date_format='day %d of %%(monthname)s') == u"day 05 of February"

def test_parse_numbers():
    assert parse_number("5,000,000", "en_US") == 5000000
    assert parse_number("5.000.000", "de") == 5000000
    assert parse_decimal("5,000,000.78781", "en_US") == 5000000.78781
    assert parse_decimal("5.000.000,78781", "de") == 5000000.78781
    assert parse_decimal("5,000,000.78781", "xx") == 5000000.78781

    try:
        parse_number("5.000.000", "en_US")
        parse_number("5.000.000,78781", "en_US")
        assert False
    except ValueError:
        assert True

def test_diff_locale_format_date():
    assert format_date(dt, "short", "en_US") == "02/05/05"
    assert format_date(dt, "short", "en_GB") == "05/02/2005"

def test_invalid_locale_format():
    assert get_abbr_weekday_names("fubar")==[]



import logging
import sha
import threading
import time

try:
    set()
except NameError:
    from sets import Set as set

from random import random
from datetime import timedelta, datetime

import cherrypy
import pkg_resources

from cherrypy.filters.basefilter import BaseFilter
from turbogears import config

log = logging.getLogger("turbogears.visit")

# Global VisitManager
_manager = None

# Global list of plugins for the Visit Tracking framework
_plugins = list()

# Accessor functions for getting and setting the current visit information.
def current():
    """Retrieve the current visit record from the cherrypy request."""
    return getattr(cherrypy.request, "tg_visit", None)

def set_current(visit):
    """Set the current visit record on the cherrypy request being processed."""
    cherrypy.request.tg_visit = visit

def _create_visit_manager(timeout):
    """Create a VisitManager based on the plugin specified in the config file."""
    plugin_name = config.get("visit.manager", "sqlobject")
    plugins = pkg_resources.iter_entry_points(
        "turbogears.visit.manager", plugin_name)
    log.debug("Loading visit manager from plugin: %s", plugin_name)
    for entrypoint in plugins:
        plugin = entrypoint.load()
        return plugin(timeout)
    raise RuntimeError("VisitManager plugin missing: %s" % plugin_name)

class MonkeyDecodingFilter(BaseFilter):

    def decode(from_enc):
        """Recursively decode all values in an iterable from specified encoding."""

        def decode_from(value, from_enc):
            if isinstance(value, dict):
                for k, v in value.items():
                    value[k] = decode_from(v, from_enc)
            elif isinstance(value, list):
                newlist = list()
                for item in value:
                    newlist.append(decode_from(item, from_enc))
                value = newlist
            elif isinstance(value, str):
                return value.decode(from_enc)
            return value

        decoded_params = decode_from(cherrypy.request.params, from_enc)
        # This is done in two steps to make sure the exception in
        # before_main can retry a decode with another encoding if needed.
        # DON'T merge those two lines.
        cherrypy.request.params = decoded_params
    decode = staticmethod(decode)

    def before_main(self):
        """Monkey patch CherryPy's decoding filter."""
        conf = cherrypy.config.get
        if not conf('decoding_filter.on', False):
            return
        if getattr(cherrypy.request, "_decoding_attempted", False):
            return
        cherrypy.request._decoding_attempted = True
        enc = conf('decoding_filter.encoding', None)
        if not enc:
            ct = cherrypy.request.headers.elements("Content-Type")
            if ct:
                ct = ct[0]
                enc = ct.params.get("charset", None)
                if not enc and ct.value.lower().startswith("text/"):
                    # http://www.w3.org/Protocols/rfc2616/rfc2616-sec3.html#sec3.7.1
                    # When no explicit charset parameter is provided by the
                    # sender, media subtypes of the "text" type are defined
                    # to have a default charset value of "ISO-8859-1" when
                    # received via HTTP.
                    enc = "ISO-8859-1"
            if not enc:
                enc = conf('decoding_filter.default_encoding', "utf-8")
        try:
            self.decode(enc)
        except UnicodeDecodeError:
            # IE and Firefox don't supply a charset when submitting form
            # params with a CT of application/x-www-form-urlencoded.
            # So after all our guessing, it could *still* be wrong.
            # Start over with ISO-8859-1, since that seems to be preferred.
            self.decode("ISO-8859-1")


# Interface for the TurboGears extension

def start_extension():
    # TODO: This should alway be active even when visit is not on,
    # we should find a better place to load this code...

    # Monkey patch CP Decoding filter
    # TODO: Is there a better way to inject this?
    # (maybe earlier than start_extension?)
    # monkey inject our replacement filter into the CP2 filter chain
    decoding_filter = MonkeyDecodingFilter()
    for index, active_filter in enumerate(
            cherrypy.filters._filterhooks.get('before_main', [])):
        if active_filter.im_class == \
                cherrypy.filters.decodingfilter.DecodingFilter:
            cherrypy.filters._filterhooks['before_main'].pop(index)
            cherrypy.filters._filterhooks['before_main'].insert(
                    index, decoding_filter.before_main)

    # Here is the real visit code.
    # Bail out if the application hasn't enabled this extension
    if not config.get("visit.on", False):
        return
    # Bail out if this extension is already running
    global _manager
    if _manager:
        return

    log.info("Visit Tracking starting")
    # How long may the visit be idle before a new visit ID is assigned?
    # The default is 20 minutes.
    timeout = timedelta(minutes=config.get("visit.timeout", 20))
    # Create the thread that manages updating the visits
    _manager = _create_visit_manager(timeout)

    visit_filter = VisitFilter()
    # Install Filter into the root filter chain
    if not hasattr(cherrypy.root, "_cp_filters"):
        cherrypy.root._cp_filters = list()
    cherrypy.root._cp_filters.append(visit_filter)

def shutdown_extension():
    # Bail out if this extension is not running.
    global _manager
    if not _manager:
        return
    log.info("Visit Tracking shutting down")
    _manager.shutdown()
    _manager = None

def create_extension_model():
    """Create the data model of the VisitManager if one exists."""
    if _manager:
        _manager.create_model()

def enable_visit_plugin(plugin):
    """Register a visit tracking plugin.

    These plugins will be called for each request.

    """
    _plugins.append(plugin)

class Visit(object):
    """Basic container for visit related data."""

    def __init__(self, key, is_new):
        self.key = key
        self.is_new = is_new


class VisitFilter(BaseFilter):
    """A filter that automatically tracks visitors."""

    def __init__(self):
        log.info("Visit filter initialized")
        get = config.get
        # Where to look for the session key in the request and in which order
        self.source = [s.strip().lower() for s in
            get("visit.source", "cookie").split(',')]
        if set(self.source).difference(('cookie', 'form')):
            log.warning("Unsupported 'visit.source' '%s' in configuration.")
        # Get the name to use for the identity cookie.
        self.cookie_name = get("visit.cookie.name", "tg-visit")
        # and the name of the request param. MUST NOT contain dashes or dots,
        # otherwise the NestedVariablesFilter will chocke on it.
        self.visit_key_param = get("visit.form.name", "tg_visit")
        # TODO: The path should probably default to whatever
        # the root is masquerading as in the event of a
        # virtual path filter.
        self.cookie_path = get("visit.cookie.path", "/")
        # The secure bit should be set for HTTPS only sites
        self.cookie_secure = get("visit.cookie.secure", False)
        # By default, I don't specify the cookie domain.
        self.cookie_domain = get("visit.cookie.domain", None)
        assert self.cookie_domain != "localhost", "localhost" \
            " is not a valid value for visit.cookie.domain. Try None instead."
        # Use max age only if the cookie shall explicitly be permanent
        self.cookie_max_age = get("visit.cookie.permanent",
            False) and int(get("visit.timeout", "20")) * 60 or None

    def before_main(self):
        """Check whether submitted request belongs to an existing visit."""
        if not config.get("visit.on", True):
            set_current(None)
            return
        cpreq = cherrypy.request
        visit = current()
        if not visit:
            visit_key = None
            for source in self.source:
                if source == 'cookie':
                    visit_key = cpreq.simple_cookie.get(self.cookie_name)
                    if visit_key:
                        visit_key = visit_key.value
                        log.debug("Retrieved visit key '%s' from cookie '%s'.",
                            visit_key, self.cookie_name)
                elif source == 'form':
                    visit_key = cpreq.params.pop(self.visit_key_param, None)
                    log.debug(
                        "Retrieved visit key '%s' from request param '%s'.",
                        visit_key, self.visit_key_param)
                if visit_key:
                    visit = _manager.visit_for_key(visit_key)
                    break
            if not visit:
                visit_key = self._generate_key()
                visit = _manager.new_visit_with_key(visit_key)
                log.debug("Created new visit with key: %s", visit_key)
            else:
                log.debug("Using visit from request with key: %s", visit_key)
            self.send_cookie(visit_key)
            set_current(visit)
        # Inform all the plugins that a request has been made for the current
        # visit. This gives plugins the opportunity to track click-path or
        # retrieve the visitor's identity.
        try:
            for plugin in _plugins:
                plugin.record_request(visit)
        except cherrypy.InternalRedirect, e:
            # Can't allow an InternalRedirect here because CherryPy is dumb,
            # instead change cherrypy.request.object_path to the url desired.
            cherrypy.request.object_path = e.path

    def _generate_key():
        """Return a (pseudo)random hash based on seed."""
        # Adding remoteHost and remotePort doesn't make this any more secure,
        # but it makes people feel secure... It's not like I check to make
        # certain you're actually making requests from that host and port. So
        # it's basically more noise.
        key_string = '%s%s%s%s' % (random(), datetime.now(),
            cherrypy.request.remote_host, cherrypy.request.remote_port)
        return sha.new(key_string).hexdigest()
    _generate_key = staticmethod(_generate_key)

    def clear_cookie(self):
        """Clear any existing visit ID cookie."""
        cookies = cherrypy.response.simple_cookie
        # clear the cookie
        log.debug("Clearing visit ID cookie")
        cookies[self.cookie_name] = ''
        cookies[self.cookie_name]['path'] = self.cookie_path
        cookies[self.cookie_name]['expires'] = ''
        cookies[self.cookie_name]['max-age'] = 0

    def send_cookie(self, visit_key):
        """Send an visit ID cookie back to the browser."""
        cookies = cherrypy.response.simple_cookie
        cookies[self.cookie_name] = visit_key
        cookies[self.cookie_name]['path'] = self.cookie_path
        if self.cookie_secure:
            cookies[self.cookie_name]['secure'] = True
        if self.cookie_domain:
            cookies[self.cookie_name]['domain'] = self.cookie_domain
        max_age = self.cookie_max_age
        if max_age:
            # use 'expires' because MSIE ignores 'max-age'
            cookies[self.cookie_name]['expires'] = time.strftime(
                "%a, %d-%b-%Y %H:%M:%S GMT",
                time.gmtime(time.time() + max_age))
            # 'max-age' takes precedence on standard conformant browsers
            # (this is better because there of no time sync issues here)
            cookies[self.cookie_name]['max-age'] = max_age
        log.debug("Sending visit ID cookie: %s",
            cookies[self.cookie_name].output())


class BaseVisitManager(threading.Thread):

    def __init__(self, timeout):
        super(BaseVisitManager, self).__init__(name="VisitManager")
        self.timeout = timeout
        self.queue = dict()
        self.lock = threading.Lock()
        self._shutdown = threading.Event()
        self.interval = 30
        self.setDaemon(True)
        # We need to create the visit model before the manager thread is
        # started.
        self.create_model()
        self.start()

    def create_model(self):
        pass

    def new_visit_with_key(self, visit_key):
        """Return a new Visit object with the given key."""
        raise NotImplementedError

    def visit_for_key(self, visit_key):
        """Return the visit for this key.

        Return None if the visit doesn't exist or has expired.

        """
        raise NotImplementedError

    def update_queued_visits(self, queue):
        """Extend the expiration of the queued visits."""
        raise NotImplementedError

    def update_visit(self, visit_key, expiry):
        try:
            self.lock.acquire()
            self.queue[visit_key] = expiry
        finally:
            self.lock.release()

    def shutdown(self, timeout=None):
        self._shutdown.set()
        self.join(timeout)
        if self.isAlive():
            log.error("Visit Manager thread failed to shutdown.")

    def run(self):
        while not self._shutdown.isSet():
            self.lock.acquire()
            queue = None
            try:
                # make a copy of the queue and empty the original
                if self.queue:
                    queue = self.queue.copy()
                    self.queue.clear()
            finally:
                self.lock.release()
            if queue is not None:
                self.update_queued_visits(queue)
            self._shutdown.wait(self.interval)



import itertools
from datetime import datetime

from turbogears import validators, expose
from turbojson import jsonify
from turbogears.widgets.base import CSSLink, JSLink, CSSSource, JSSource, \
                                    Widget, WidgetsList, static, mochikit, \
                                    CoreWD
from turbogears.widgets.i18n import CalendarLangFileLink
from turbogears.widgets.forms import FormField, CompoundFormField, TextField, \
                                    HiddenField, TableForm, CheckBox, \
                                    RadioButtonList
from turbogears.widgets.rpc import RPC

__all__ = ["CalendarDatePicker", "CalendarDateTimePicker",
    "AutoCompleteField", "AutoCompleteTextField",
    "LinkRemoteFunction", "RemoteForm", "AjaxGrid", "URLLink"]


class CalendarDatePicker(FormField):
    """Use a Javascript calendar system to allow picking of calendar dates."""

    template = """
    <div xmlns:py="http://purl.org/kid/ns#">
    <input type="text" id="${field_id}" class="${field_class}" name="${name}" value="${strdate}" py:attrs="attrs"/>
    <input type="button" id="${field_id}_trigger" class="date_field_button" value="${button_text}"/>
    <script type="text/javascript">
    Calendar.setup({
        inputField : "${field_id}",
        ifFormat : "${format}",
        button : "${field_id}_trigger"
        <span py:if="picker_shows_time" py:replace="', showsTime : true'"/>
    });
    </script>
    </div>
    """
    params = ["attrs", "skin", "picker_shows_time", "button_text",
        "format", "calendar_lang"]
    params_doc = {'attrs': 'Extra attributes',
        'skin': 'For alternate skins, such as "calendar-blue" or "skins/aqua/theme"',
        'picker_shows_time': 'Whether the calendar should let you pick a time, too',
        'button_text': 'Text for the button that will show the calendar picker',
        'format': 'The date format (default is mm/dd/yyyy)',
        'calendar_lang': 'The language to be used in the calendar picker.'}
    attrs = {}
    skin = "calendar-system"
    picker_shows_time = False
    button_text = "Choose"
    format = "%m/%d/%Y"
    calendar_lang = None
    _default = None

    def __init__(self, name=None, default=None, not_empty=True,
                 calendar_lang=None, validator=None, format=None, **kw):
        super(CalendarDatePicker, self).__init__(name, **kw)
        self.not_empty = not_empty
        if default is not None or not self.not_empty:
            self._default = default
        if format is not None:
            self.format = format
        if validator is None:
            self.validator = validators.DateTimeConverter(
                format=self.format, not_empty=self.not_empty)
        else:
            self.validator = validator
        if calendar_lang:
            self.calendar_lang = calendar_lang
        javascript = [JSLink(static, "calendar/calendar.js"),
            JSLink(static, "calendar/calendar-setup.js")]
        javascript.append(CalendarLangFileLink(static,
            language=self.calendar_lang))
        self.javascript = self.javascript + javascript
        if self.skin:
            css = [CSSLink(static, "calendar/%s.css" % self.skin)]
            self.css = self.css + css

    def _get_default(self):
        if self._default is None and self.not_empty:
            return datetime.now()
        return self._default
    default = property(_get_default)

    def update_params(self, d):
        super(CalendarDatePicker, self).update_params(d)
        if hasattr(d['value'], 'strftime'):
            d['strdate'] = d['value'].strftime(d['format'])
        else:
            d['strdate'] = d['value']


class CalendarDatePickerDesc(CoreWD):

    name = "Calendar"
    for_widget = CalendarDatePicker("date_picker")


class CalendarDateTimePicker(CalendarDatePicker):
    """Javascript calendar system to allow picking of dates with times."""

    format = "%Y/%m/%d %H:%M"
    picker_shows_time = True


class CalendarDateTimePickerDesc(CoreWD):

    name = "Calendar with time"
    for_widget = CalendarDateTimePicker("datetime_picker")


class AutoComplete(Widget):
    """Mixin class for autocomplete fields.

    Performs Ajax-style autocompletion by requesting search
    results from the server as the user types.

    """
    javascript = [mochikit, JSLink(static,"autocompletefield.js")]
    css = [CSSLink(static,"autocompletefield.css")]
    params = ["search_controller", "search_param", "result_name", "attrs",
        "only_suggest", "complete_delay", "take_focus", "min_chars", "show_spinner"]
    params_doc = {'attrs': 'Extra attributes',
        'search_controller': 'Name of the controller returning the auto completions',
        'search_param': 'Name of the search parameter ("*" passes all form fields)',
        'result_name': 'Name of the result list returned by the controller',
        'only_suggest': 'If true, pressing enter does not automatically submit the first list item.',
        'complete_delay': 'Delay (in seconds) before loading new auto completions',
        'take_focus': 'If true, take focus on load.',
        'min_chars': 'Minimum number of characters to type before autocomplete activates',
        'show_spinner': 'If false, the spinner (load indicator) is not shown.'}
    attrs = {}
    search_controller = ""
    search_param = "searchString"
    result_name = "textItems"
    only_suggest = False
    complete_delay = 0.200
    take_focus = False
    min_chars = 1
    show_spinner = True


class AutoCompleteField(CompoundFormField, AutoComplete):
    """Text field with auto complete functionality and hidden key field."""

    template = """
    <div xmlns:py="http://purl.org/kid/ns#" id="${field_id}">
    <script language="JavaScript" type="text/JavaScript">
        AutoCompleteManager${field_id} = new AutoCompleteManager('${field_id}',
        '${text_field.field_id}', '${hidden_field.field_id}',
        '${search_controller}', '${search_param}', '${result_name}',${str(only_suggest).lower()},
        '${show_spinner and tg.url([tg.widgets, 'turbogears.widgets/spinner.gif']) or None}',
        ${complete_delay}, ${str(take_focus).lower()}, ${min_chars});
        addLoadEvent(AutoCompleteManager${field_id}.initialize);
    </script>
    ${text_field.display(value_for(text_field), **params_for(text_field))}
    <img py:if="show_spinner" id="autoCompleteSpinner${field_id}"
        src="${tg.url([tg.widgets, 'turbogears.widgets/spinnerstopped.png'])}" alt=""/>
    <div class="autoTextResults" id="autoCompleteResults${field_id}"/>
    ${hidden_field.display(value_for(hidden_field), **params_for(hidden_field))}
    </div>
    """

    member_widgets = ["text_field", "hidden_field"]
    text_field = TextField(name="text")
    hidden_field = HiddenField(name="hidden")


class AutoCompleteFieldDesc(CoreWD):

    name = "AutoCompleteField"

    codes = """AK AL AR AS AZ CA CO CT DC DE FL FM GA GU HI IA ID IL IN KS
        KY LA MA MD ME MH MI MN MO MP MS MT NC ND NE NH NJ NM NV NY OH
        OK OR PA PR PW RI SC SD TN TX UM UT VA VI VT WA WI WV WY""".split()

    states = """Alaska Alabama Arkansas American_Samoa Arizona
        California Colorado Connecticut District_of_Columbia
        Delaware Florida Federated_States_of_Micronesia Georgia Guam
        Hawaii Iowa Idaho Illinois Indiana Kansas Kentucky Louisiana
        Massachusetts Maryland Maine Marshall_Islands Michigan
        Minnesota Missouri Northern_Mariana_Islands Mississippi
        Montana North_Carolina North_Dakota Nebraska New_Hampshire
        New_Jersey New_Mexico Nevada New_York Ohio Oklahoma Oregon
        Pennsylvania Puerto_Rico Palau Rhode_Island South_Carolina
        South_Dakota Tennessee Texas U.S._Minor_Outlying_Islands
        Utah Virginia Virgin_Islands_of_the_U.S. Vermont Washington
        Wisconsin West_Virginia Wyoming""".split()
    states = map(lambda s: s.replace('_', ' '), states)

    state_code = dict(zip(codes, states))

    template = """
    <form xmlns:py="http://purl.org/kid/ns#" onsubmit="if (
        this.elements[0].value &amp;&amp; this.elements[1].value)
        alert('The alpha code for '+this.elements[0].value
        +' is '+this.elements[1].value+'.');return false"><table>
        <tr><th>State</th><td py:content="for_widget.display()"/>
        <td><input type="submit" value="Show alpha code"/></td></tr>
    </table></form>
    """

    full_class_name = "turbogears.widgets.AutoCompleteField"

    def __init__(self, *args, **kw):
        super(AutoCompleteFieldDesc, self).__init__(*args, **kw)
        self.for_widget = AutoCompleteField(name="state_and_code",
            search_controller="%s/search" % self.full_class_name,
            search_param="state", result_name="states")

    [expose(format="json")]
    def search(self, state):
        states = []
        code = state.upper()
        if code in self.state_code:
            states.append((self.state_code[code], code))
        else:
            states.extend([s for s  in zip(self.states, self.codes)
                if s[0].lower().startswith(state.lower())])
        return dict(states=states)


class AutoCompleteTextField(TextField, AutoComplete):
    """Text field with auto complete functionality."""

    template = """
    <div xmlns:py="http://purl.org/kid/ns#">
    <script language="JavaScript" type="text/JavaScript">
        AutoCompleteManager${field_id} = new AutoCompleteManager('${field_id}', '${field_id}', null,
        '${search_controller}', '${search_param}', '${result_name}', ${str(only_suggest).lower()},
        '${show_spinner and tg.url([tg.widgets, 'turbogears.widgets/spinner.gif']) or None}',
        ${complete_delay}, ${str(take_focus).lower()}, ${min_chars});
        addLoadEvent(AutoCompleteManager${field_id}.initialize);
    </script>
    <input type="text" name="${name}" class="${field_class}" id="${field_id}"
        value="${value}" py:attrs="attrs"/>
    <img py:if="show_spinner" id="autoCompleteSpinner${field_id}"
        src="${tg.url([tg.widgets, 'turbogears.widgets/spinnerstopped.png'])}" alt=""/>
    <div class="autoTextResults" id="autoCompleteResults${field_id}"/>
    </div>
    """


class AutoCompleteTextFieldDesc(CoreWD):

    name = "AutoCompleteTextField"

    states = AutoCompleteFieldDesc.states
    state_code = AutoCompleteFieldDesc.state_code

    template = """
    <table xmlns:py="http://purl.org/kid/ns#">
        <tr><th>State</th><td py:content="for_widget.display()"/></tr>
    </table>
    """

    full_class_name = "turbogears.widgets.AutoCompleteTextField"

    def __init__(self, *args, **kw):
        super(AutoCompleteTextFieldDesc, self).__init__(*args, **kw)
        self.for_widget = AutoCompleteTextField(name="state_only",
            search_controller="%s/search" % self.full_class_name,
            search_param="state", result_name="states")

    [expose(format="json")]
    def search(self, state):
        states = []
        code = state.upper()
        if code in self.state_code:
            states.append(self.state_code[code])
        else:
            states.extend([s for s  in self.states
                if s.lower().startswith(state.lower())])
        return dict(states=states)


class LinkRemoteFunction(RPC):
    """Link with remote execution.

    Returns a link that executes a POST asynchronously
    and updates a DOM Object with the result of it.

    """
    template = """
    <a xmlns:py="http://purl.org/kid/ns#" name="${name}"
        py:attrs="attrs" py:content="value" onclick="${js}" href="#"/>
    """

    params = ["attrs"]
    attrs = {}


class LinkRemoteFunctionDesc(CoreWD):

    name = "AJAX remote function"

    states = AutoCompleteFieldDesc.states

    template = """
    <div id="items">
        ${for_widget.display("States starting with the letter 'N'", update="items")}
    </div>
    """

    full_class_name = "turbogears.widgets.LinkRemoteFunction"

    def __init__(self, *args, **kw):
        super(LinkRemoteFunctionDesc, self).__init__(*args, **kw)
        self.for_widget = LinkRemoteFunction(
            name="linkrf", action="%s/search_linkrf" % self.full_class_name,
            data = dict(state_starts_with="N"))

    [expose()]
    def search_linkrf(self, state_starts_with):
        return '<br/>'.join(filter(
            lambda item: item.startswith(state_starts_with), self.states))


class RemoteForm(RPC, TableForm):
    """AJAX table form.

    A TableForm that submits the data asynchronously and loads the resulting
    HTML into a DOM object

    """
    def update_params(self, d):
        super(RemoteForm, self).update_params(d)
        d['form_attrs']['onSubmit'] = "return !remoteFormRequest(this, '%s', %s);" % (
            d.get("update", ''), jsonify.encode(self.get_options(d)))


class RemoteFormDesc(CoreWD):

    name = "AJAX Form"

    class TestFormFields(WidgetsList):
        name = TextField()
        age = TextField()
        check = CheckBox()
        radio = RadioButtonList(options=list(enumerate(
            "Python Java Pascal Ruby".split())), default=3)

    template = """
    <div>
        ${for_widget.display()}
        <div id="post_data">&nbsp;</div>
    </div>
    """
    full_class_name = "turbogears.widgets.RemoteForm"

    def __init__(self, *args, **kw):
        super(RemoteFormDesc, self).__init__(*args, **kw)
        self.for_widget = RemoteForm(
            fields = self.TestFormFields(),
            name="remote_form",
            update = "post_data",
            action = "%s/post_data_rf" % self.full_class_name,
            before = "alert('pre-hook')",
            confirm = "Confirm?",
        )

    [expose()]
    def post_data_rf(self, **kw):
        return """Received data:<br/>%r""" % kw


ajaxgridcounter = itertools.count()

class AjaxGrid(Widget):
    """AJAX updateable datagrid based on widget.js grid"""

    template = """<div id="${id}" xmlns:py="http://purl.org/kid/ns#">
    <a py:if="refresh_text"
       href="#"
       onclick="javascript:${id}_AjaxGrid.refresh(${defaults});return false;">
       ${refresh_text}
    </a>
    <div id="${id}_update"></div>
    <script type="text/javascript">
        addLoadEvent(partial(${id}_AjaxGrid.refresh, ${defaults}));
    </script>
    </div>
    """

    params = ["refresh_text", "id", "defaults"]

    defaults = {}
    refresh_text = "Update"
    id = "ajaxgrid_%d" % ajaxgridcounter.next()

    def __init__(self, refresh_url, *args, **kw):
        super(AjaxGrid, self).__init__(*args, **kw)
        target = "%s_update" % self.id
        self.javascript = [
            mochikit,
            JSLink("turbogears", "js/widget.js"),
            JSLink(static, "ajaxgrid.js"),
            JSSource("""
                %(id)s_AjaxGrid = new AjaxGrid('%(refresh_url)s', '%(target)s');
            """ % dict(id=self.id, refresh_url=refresh_url, target=target)
            ),
        ]

    def update_params(self, d):
        super(AjaxGrid, self).update_params(d)
        d["defaults"] = jsonify.encode(d["defaults"])


class AjaxGridDesc(CoreWD):

    name = "AJAX Grid"

    def facgen(n):
        total = 1
        yield 0, 1
        for x in xrange(1, n+1):
            total *= x
            yield x, total
    facgen = staticmethod(facgen)


    full_class_name = "turbogears.widgets.AjaxGrid"

    def __init__(self, *args, **kw):
        super(AjaxGridDesc, self).__init__(*args, **kw)
        self.for_widget = AjaxGrid(
            refresh_url = "%s/update" % self.full_class_name,
            # Dummy default params, just POC
            defaults = dict(),
        )
        self.update_count = itertools.count()

    [expose(format="json")]
    def update(self):
        return dict(
            headers = ["N", "fact(N)"],
            rows = list(self.facgen(self.update_count.next())),
        )


class URLLink(FormField):
    """Hyperlink"""

    template = """
    <a xmlns:py="http://purl.org/kid/ns#"
       href="$link"
       target="$target"
       py:attrs="attrs"
    >$text</a>
    """

    params = ["target", "text", "link", "attrs"]
    attrs = {}
    params_doc = {'link': 'Hyperlink',
                  'target': 'Specify where the link should be opened',
                  'text': 'The message to be shown for the link',
                  'attrs': 'Extra attributes'}



# Formatting configuration for locale sq

languages={'sq': 'shqipe'}

countries={'AL': u'Shqip\xebria'}

months=['janar', 'shkurt', 'mars', 'prill', 'maj', 'qershor', 'korrik', 'gusht', 'shtator', 'tetor', u'n\xebntor', 'dhjetor']

abbrMonths=['Jan', 'Shk', 'Mar', 'Pri', 'Maj', 'Qer', 'Kor', 'Gsh', 'Sht', 'Tet', u'N\xebn', 'Dhj']

days=[u'e h\xebn\xeb', u'e mart\xeb', u'e m\xebrkur\xeb', 'e enjte', 'e premte', u'e shtun\xeb', 'e diel']

abbrDays=[u'H\xebn', 'Mar', u'M\xebr', 'Enj', 'Pre', 'Sht', 'Die']

dateFormats={'medium': '%Y-%m-%d', 'full': '%%(dayname)s, %d %%(monthname)s %Y', 'long': '%d %%(monthname)s %Y', 'short': '%y-%m-%d'}

numericSymbols={'group': '.', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': ',', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}



# Formatting configuration for locale id_ID

languages={'gv': 'Manx', 'gu': 'Gujarati', 'alg': 'Bahasa Algonquia', 'afa': 'Afro-Asiatik (Lainnya)', 'gd': 'Gaelik Skotlandia', 'ga': 'Irlandia', 'gn': 'Guarani', 'gl': 'Gallegan', 'lg': 'Ganda', 'gmh': 'Jerman, Abad Pertengahan (kl.1050-1500)', 'lb': 'Luxembourg', 'la': 'Latin', 'ln': 'Lingala', 'lo': 'Lao', 'tt': 'Tatar', 'tr': 'Turkish', 'ts': 'Tsonga', 'li': 'Limburg', 'lv': 'Latvian', 'aus': 'Bahasa-bahasa Australia', 'lu': 'Luba-Katanga', 'tk': 'Turkmen', 'th': 'Thai', 'sr': 'Serbian', 'tg': 'Tajik', 'te': 'Telugu', 'grc': 'Yunani Kuno (sd 1453)', 'yi': 'Yiddish', 'mga': 'Irlandia Abad Pertengahan (900-1200)', 'yo': 'Yoruba', 'de': 'Jerman', 'da': 'Denmark', 'za': 'Zhuang', 'ta': 'Tamil', 'dz': 'Dzongkha', 'rm': 'Rhaeto-Romance', 'dv': 'Divehi', 'gem': 'Jermanik (Lainnya)', 'bat': 'Baltik (Lainnya)', 'tl': 'Tagalog', 'qu': 'Quechua', 'ml': 'Malayalam', 'ban': 'Balin', 'bai': 'Bahasa-bahasa Bamileke', 'el': 'Yunani', 'eo': 'Esperanto', 'en': 'Inggris', 'art': 'Buatan (Lainnya)', 'ee': 'Ewe', 'phi': 'Filipina (Lainnya)', 'mh': 'Marshall', 'arc': 'Aram', 'mk': 'Macedonian', 'eu': 'Basque', 'et': 'Estonian', 'arn': 'Araucan', 'ru': 'Russian', 'rw': 'Kinyarwanda', 'goh': 'Jerman Kuno (kl.750-1050)', 'peo': 'Persia Kuno (kl.600-400 SM.)', 'got': 'Gothik', 'rn': 'Rundi', 'ro': 'Romanian', 'jv': 'Jawa', 'sux': 'Sumeria', 'be': 'Belarusia', 'bg': 'Bulgaria', 'ba': 'Bashkir', 'wa': 'Walloon', 'ast': 'Astur', 'wo': 'Wolof', 'bm': 'Bambara', 'bn': 'Bengal', 'bo': 'Tibet', 'bh': 'Bihari', 'bi': 'Bislama', 'br': 'Breton', 'bs': 'Bosnia', 'lez': 'Lezghia', 'ja': 'Japanese', 'cop': 'Koptik', 'om': 'Oromo', 'oj': 'Ojibwa', 'ace': 'Aceh', 'syr': 'Syria', 'ty': 'Tahitian', 'ath': 'Bahasa-bahasa Athapaska', 'akk': 'Akkadien', 'tw': 'Twi', 'os': 'Ossetic', 'or': 'Oriya', 'xh': 'Xhosa', 'ch': 'Chamorro', 'co': 'Korsika', 'jrb': 'Judeo-Arab', 'ca': 'Catalan', 'ce': 'Chechen', 'cy': 'Welsh', 'ang': 'Inggris Kuno (kl.450-1100)', 'cs': 'Ceko', 'cr': 'Cree', 'apa': 'Bahasa-bahasa Apache', 'cv': 'Chuvash', 've': 'Venda', 'ps': 'Pashto (Pushto)', 'kok': 'Konkani', 'pt': 'Portugis', 'chk': 'Chuuke', 'fro': 'Perancis Kuno (842-kl.1400)', 'lt': 'Lithuania', 'frm': 'Perancis, Abad Pertengahan (kl.1400-1600)', 'mkh': 'Mon-Khmer (Lainnya)', 'paa': 'Papuan (Lainnya)', 'pa': 'Punjabi', 'vi': 'Vietnamese', 'kos': 'Kosrae', 'gil': 'Gilbert', 'is': 'Icelandic', 'pl': 'Polish', 'hz': 'Herero', 'zh': 'Cina', 'hy': 'Armenia', 'hr': 'Kroasia', 'ti': 'Tigrinya', 'ka': 'Georgian', 'hu': 'Hungaria', 'hi': 'Hindi', 'ho': 'Hiri Motu', 'ha': 'Hausa', 'bug': 'Bugis', 'he': 'Ibrani', 'mg': 'Malagasi', 'pi': 'Pali', 'fur': 'Friuli', 'uz': 'Uzbek', 'car': 'Karib', 'mo': 'Moldavian', 'mn': 'Mongolian', 'mi': 'Maori', 'ik': 'Inupiaq', 'cau': 'Kaukasia (Lainnya)', 'ur': 'Urdu', 'mt': 'Maltese', 'cai': 'India Amerika Tengah (Lainnnya)', 'ms': 'Malay', 'mr': 'Marathi', 'ug': 'Uighur', 'haw': 'Hawaii', 'my': 'Burma', 'sq': 'Albanian', 'aa': 'Afar', 'ab': 'Abkhaz', 'ae': 'Avestan', 'ss': 'Swati', 'af': 'Afrikaans', 'tn': 'Tswana', 'ne': 'Nepal', 'ak': 'Akan', 'am': 'Amharik', 'it': 'Italian', 'sv': 'Swedia', 'ii': 'Sichuan Yi', 'ia': 'Interlingua', 'as': 'Assam', 'ar': 'Arab', 'su': 'Sundan', 'io': 'Ido', 'av': 'Avarik', 'ay': 'Aymara', 'az': 'Azerbaijan', 'ie': 'Interlingue', 'id': 'Bahasa Indonesia', 'ig': 'Igbo', 'sk': 'Slovak', 'jpr': 'Judeo-Persia', 'nl': 'Belanda', 'nn': 'Norwegian Nynorsk', 'no': 'Norwegian', 'na': 'Nauru', 'nb': u'Norwegian Bokm\xe5l', 'enm': 'Inggris, Abad Pertengahan (1100-1500)', 'ng': 'Ndonga', 'ny': 'Nyanja; Chichewa; Chewa', 'vo': u'Volap\xfck', 'es': 'Spanyol', 'zu': 'Zulu', 'so': 'Somali', 'mis': 'Bahasa Lain-lain', 'nv': 'Navajo', 'sn': 'Shona', 'map': 'Austronesia', 'fr': 'Perancis', 'cel': 'Celtic (Lainnya)', 'sm': 'Samoan', 'fy': 'Frisi', 'fa': 'Persia', 'mad': 'Madura', 'ff': 'Fulah', 'fi': 'Finlandia', 'fj': 'Fiji', 'sa': 'Sanskrit', 'uk': 'Ukrainian', 'fo': 'Faro', 'mak': 'Makassar', 'egy': 'Mesir Kuno', 'kg': 'Kongo', 'kk': 'Kazakh', 'kj': 'Kuanyama', 'ki': 'Kikuyu', 'ko': 'Korea', 'kn': 'Kannada', 'km': 'Khmer', 'kl': 'Kalaallisut', 'ks': 'Kashmir', 'kr': 'Kanuri', 'si': 'Sinhalese', 'sh': 'Serbo-Croatian', 'kw': 'Cornish', 'kv': 'Komi', 'ku': 'Kurdi', 'sl': 'Slovenian', 'sc': 'Sardinian', 'ky': 'Kirghiz', 'sg': 'Sango', 'sw': 'Swahili', 'se': 'Northern Sami', 'sd': 'Sindhi'}

countries={'BD': 'Bangladesh', 'BE': 'Belgia', 'BF': 'Burkina Faso', 'BG': 'Bulgaria', 'BA': 'Bosnia dan Herzegovina', 'BB': 'Barbados', 'WF': 'Wallis dan Futuna', 'BM': 'Bermuda', 'BN': 'Brunei', 'BO': 'Bolivia', 'BH': 'Bahrain', 'BI': 'Burundi', 'BJ': 'Benin', 'BT': 'Bhutan', 'JM': 'Jamaika', 'BV': 'Kepulauan Bouvet', 'BW': 'Botswana', 'WS': 'Samoa', 'BR': 'Brazil', 'BS': 'Bahamas', 'BY': 'Belarusia', 'BZ': 'Belize', 'RU': 'Rusia', 'RW': 'Rwanda', 'RE': u'R\xe9union', 'TM': 'Turkmenistan', 'TJ': 'Tajikistan', 'RO': 'Romania', 'TK': 'Tokelau', 'GW': 'Guinea-Bissau', 'GU': 'Guam', 'GT': 'Guatemala', 'GS': 'Georgia Selatan dan Kepulauan Sandwich Selatan', 'GR': 'Yunani', 'GQ': 'Guinea Khatulistiwa', 'GP': 'Guadeloupe', 'JP': 'Jepang', 'GY': 'Guyana', 'GF': 'Guyana Perancis', 'GE': 'Georgia', 'GD': 'Grenada', 'GB': 'Inggris Raya', 'GA': 'Gabon', 'SV': 'El Salvador', 'GN': 'Guinea', 'GM': 'Gambia', 'GL': 'Greenland', 'GI': 'Gibraltar', 'GH': 'Ghana', 'OM': 'Oman', 'TN': 'Tunisia', 'JO': 'Yordania', 'SP': 'Serbia', 'HR': 'Kroasia', 'HT': 'Haiti', 'HU': 'Hungaria', 'HK': 'Hong Kong S.A.R., Cina', 'HN': 'Honduras', 'HM': 'Pulau Heard dan Kepulauan McDonald', 'VE': 'Venezuela', 'PR': 'Puerto Riko', 'PS': 'Otoritas Palestina', 'PW': 'Palau', 'PT': 'Portugis', 'SJ': 'Svalbard dan Jan Mayen', 'PY': 'Paraguay', 'IQ': 'Iraq', 'PA': 'Panama', 'PF': 'Polynesia Perancis', 'PG': 'Papua Nugini', 'PE': 'Peru', 'PK': 'Pakistan', 'PH': 'Filipina', 'PN': 'Pitcairn', 'PL': 'Polandia', 'PM': 'Saint Pierre dan Miquelon', 'ZM': 'Zambia', 'EH': 'Sahara Barat', 'EE': 'Estonia', 'EG': 'Mesir', 'ZA': 'Afrika Selatan', 'EC': 'Ekuador', 'IT': 'Itali', 'VN': 'Vietnam', 'SB': 'Kepulauan Solomon', 'ET': 'Ethiopia', 'SO': 'Somalia', 'ZW': 'Zimbabwe', 'SA': 'Arab Saudi', 'ES': 'Spanyol', 'ER': 'Eritrea', 'MD': 'Moldova', 'MG': 'Madagaskar', 'MA': 'Maroko', 'MC': 'Monaco', 'UZ': 'Uzbekistan', 'MM': 'Myanmar', 'ML': 'Mali', 'MO': 'Makao S.A.R. Cina', 'MN': 'Mongolia', 'MH': 'Kepulauan Marshall', 'MK': 'Macedonia', 'MU': 'Mauritius', 'MT': 'Malta', 'MW': 'Malawi', 'MV': 'Maldives', 'MQ': 'Martinique', 'MP': 'Kepualuan Mariana Utara', 'MS': 'Montserrat', 'MR': 'Mauritania', 'UG': 'Uganda', 'MY': 'Malaysia', 'MX': 'Mexico', 'IL': 'Israel', 'FR': 'Perancis', 'SH': 'Saint Helena', 'FI': 'Finlandia', 'FJ': 'Fiji', 'FK': 'Kepulauan Falkland', 'FM': 'Mikronesia', 'FO': 'Kepulauan Faroe', 'NI': 'Nicaragua', 'NL': 'Netherlands', 'NO': 'Norwegia', 'NA': 'Namibia', 'VU': 'Vanuatu', 'NC': 'Kaledonia Baru', 'NE': 'Niger', 'NF': 'Kepulauan Norfolk', 'NG': 'Nigeria', 'NZ': 'Selandia Baru', 'NP': 'Nepal', 'NR': 'Nauru', 'NU': 'Niue', 'CK': 'Kepulauan Cook', 'CI': 'Pantai Gading', 'CH': 'Swiss', 'CO': 'Kolombia', 'CN': 'Cina', 'CM': 'Kamerun', 'CL': 'Chili', 'CC': 'Kepulauan Cocos', 'CA': 'Kanada', 'CG': 'Kongo', 'CF': 'Republik Afrika Tengah', 'CD': 'Republik Demokratik Kongo', 'CZ': 'Republik Ceko', 'CY': 'Siprus', 'CX': 'Pulau Christmas', 'CR': 'Kosta Rika', 'CV': 'Tanjung Verde', 'CU': 'Kuba', 'SZ': 'Swaziland', 'SY': 'Syria', 'KG': 'Kyrgyzstan', 'KE': 'Kenya', 'SR': 'Suriname', 'KI': 'Kiribati', 'KH': 'Kamboja', 'KN': 'Saint Kitts dan Nevis', 'KM': 'Komoros', 'ST': 'Sao Tome dan Principe', 'SK': 'Slovakia', 'KR': 'Korea Selatan', 'SI': 'Slovenia', 'KP': 'Korea Utara', 'KW': 'Kuwait', 'SN': 'Senegal', 'SM': 'San Marino', 'SL': 'Sierra Leone', 'SC': 'Seychelles', 'KZ': 'Kazakhstan', 'KY': 'Kepulauan Kayman', 'SG': 'Singapura', 'SE': 'Sweden', 'SD': 'Sudan', 'DO': 'Republik Dominika', 'DM': 'Dominika', 'DJ': 'Jibouti', 'DK': 'Denmark', 'VG': 'Kepulauan British Virgin', 'DE': 'Jerman', 'YE': 'Yaman', 'DZ': 'Algeria', 'US': 'Amerika Serikat', 'UY': 'Uruguay', 'YU': 'Yugoslavia', 'YT': 'Mayotte', 'LB': 'Lebanon', 'LC': 'Santa Lusia', 'LA': 'Laos', 'TV': 'Tuvalu', 'TW': 'Taiwan', 'TT': 'Trinidad dan Tobago', 'TR': 'Turkey', 'LK': 'Sri Lanka', 'LI': 'Liechtenstein', 'LV': 'Latvia', 'TO': 'Tonga', 'LT': 'Lithuania', 'LU': 'Luxembourg', 'LR': 'Liberia', 'LS': 'Lesotho', 'TH': 'Thailand', 'TG': 'Togo', 'TD': 'Chad', 'LY': 'Libya', 'VA': 'Vatikan', 'VC': 'Saint Vincent dan Grenadines', 'AE': 'Uni Emirat Arab', 'AD': 'Andora', 'AG': 'Antigua dan Barbuda', 'AF': 'Afghanistan', 'AI': 'Anguilla', 'VI': 'Kepulauan U.S. Virgin', 'IS': 'Islandia', 'IR': 'Iran', 'AM': 'Armenia', 'AL': 'Albania', 'AO': 'Angola', 'AN': 'Antilles Belanda', 'AQ': 'Antarktika', 'AS': 'Samoa Amerika', 'AR': 'Argentina', 'AU': 'Australia', 'AT': 'Austria', 'AW': 'Aruba', 'IN': 'India', 'TZ': 'Tanzania', 'AZ': 'Azerbaijan', 'IE': 'Irlandia', 'ID': 'Indonesia', 'UA': 'Ukraina', 'QA': 'Qatar', 'MZ': 'Mozambique'}

months=['Januari', 'Februari', 'Maret', 'April', 'Mei', 'Juni', 'Juli', 'Agustus', 'September', 'Oktober', 'November', 'Desember']

abbrMonths=['Jan', 'Feb', 'Mar', 'Apr', 'Mei', 'Jun', 'Jul', 'Agu', 'Sep', 'Okt', 'Nov', 'Des']

days=['Senin', 'Selasa', 'Rabu', 'Kamis', 'Jumat', 'Sabtu', 'Minggu']

abbrDays=['Sen', 'Sel', 'Rab', 'Kam', 'Jum', 'Sab', 'Min']

dateFormats={'medium': '%d %%(abbrmonthname)s %y', 'full': '%%(dayname)s %d %%(monthname)s %Y', 'long': '%d %%(monthname)s %Y', 'short': '%d/%m/%y'}

numericSymbols={'group': '.', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': ',', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}



from unittest import TestCase
from datetime import datetime
import cherrypy
from turbogears import widgets, config, controllers, expose, mochikit, \
    validate, validators, testutil


class MyFormFields(widgets.WidgetsList):
    #XXX: Since allow_extra_fields should be removed from validators.Schema,
    #     we need a validator for every input-expecting widget
    name = widgets.TextField(validator=validators.String())
    age = widgets.TextField(validator=validators.Int(), default=0)
    date = widgets.CalendarDatePicker(validator=validators.DateConverter(
                                            if_empty=datetime.now()))

myfields = MyFormFields()
myform = widgets.TableForm(fields=myfields)


class MyRoot(controllers.RootController):

    [expose(template="turbogears.tests.form")]
    def index(self):
        return dict(form=myform)

    [expose(template="turbogears.tests.form")]
    def fields(self):
        myfields.display = lambda **kw: kw.values()
        return dict(form=myfields)

    [expose(template="turbogears.tests.form")]
    def usemochi(self):
        return dict(mochi=mochikit, form=myform)

    [expose(template="turbogears.tests.othertemplate")]
    [validate(form=myform)]
    def testform(self, name, date, age, tg_errors=None):
        if tg_errors:
            self.has_errors = True
        self.name = name
        self.age = age
        self.date = date

    [expose()]
    [validate(form=myform)]
    def testform_new_style(self, name, date, age):
        if cherrypy.request.validation_errors:
            self.has_errors = True
        self.name = name
        self.age = age
        self.date = date

def test_form_translation():
    """Form input is translated into properly converted parameters"""
    root = MyRoot()
    cherrypy.root = root
    testutil.create_request("/testform?name=ed&date=11/05/2005&age=5")
    assert root.name == "ed"
    assert root.age == 5

def test_form_translation_new_style():
    """Form input is translated into properly converted parameters"""
    root = MyRoot()
    cherrypy.root = root
    testutil.create_request("/testform_new_style?name=ed&date=11/05/2005&age=5&")
    assert root.name == "ed"
    assert root.age == 5

def test_invalid_form_with_error_handling():
    """Invalid forms can be handled by the method"""
    root = cherrypy.root
    testutil.create_request("/testform?name=ed&age=edalso&date=11/05/2005")
    assert root.has_errors

def test_css_should_appear():
    """CSS should appear when asked for"""
    testutil.create_request("/")
    assert "calendar-system.css" in cherrypy.response.body[0]
    testutil.create_request("/fields")
    assert "calendar-system.css" in cherrypy.response.body[0]

def test_javascript_should_appear():
    """JavaScript should appear when asked for"""
    testutil.create_request("/")
    assert "calendar.js" in cherrypy.response.body[0]
    testutil.create_request("/fields")
    assert "calendar.js" in cherrypy.response.body[0]

def test_include_mochikit():
    """JSLinks (and MochiKit especially) can be included easily"""
    testutil.create_request("/usemochi")
    assert "MochiKit.js" in cherrypy.response.body[0]

def test_suppress_mochikit():
    """MochiKit inclusion can be suppressed"""
    config.update({"global": {"tg.mochikit_suppress": True}})
    testutil.create_request("/usemochi")
    suppressed_body = cherrypy.response.body[0]
    # repair the fixture
    config.update({"global": {"tg.mochikit_suppress": False}})
    testutil.create_request("/usemochi")
    included_body = cherrypy.response.body[0]
    assert "MochiKit.js" not in suppressed_body
    assert "MochiKit.js" in included_body

def test_mochikit_everywhere():
    """MochiKit can be included everywhere by setting tg.mochikit_all"""
    config.update({"global": {"tg.mochikit_all": True}})
    testutil.create_request("/")
    config.update({"global": {"tg.mochikit_all": False}})
    assert "MochiKit.js" in cherrypy.response.body[0]

def test_mochikit_nowhere():
    """Setting tg.mochikit_suppress will prevent including it everywhere"""
    config.update({"global": {"tg.mochikit_all": True}})
    config.update({"global": {"tg.mochikit_suppress": True}})
    testutil.create_request("/")
    config.update({"global": {"tg.mochikit_all": False}})
    config.update({"global": {"tg.mochikit_suppress": False}})
    assert "MochiKit.js" not in cherrypy.response.body[0]

def test_include_widgets():
    """Any widget can be included everywhere by setting tg.include_widgets"""
    config.update({"global": {"tg.include_widgets": ["mochikit"]}})
    testutil.create_request("/")
    config.update({"global": {"tg.include_widgets": []}})
    assert "MochiKit.js" in cherrypy.response.body[0]


class State(object):
    counter = 0

class AddingValidator(validators.FancyValidator):
    def _to_python(self, value, state=None):
        state.counter += 1
        return value

class AddingSchema(validators.Schema):
    a = AddingValidator()
    b = AddingValidator()

class AddingNestedSchema(AddingSchema):
    c = AddingSchema()


class TestValidationState(TestCase):

    class Controller(controllers.RootController):

        [expose()]
        [validate(validators=AddingNestedSchema(), state_factory=State)]
        def validation(self, a, b, c):
            return 'counter: %d' % cherrypy.request.validation_state.counter

    def __init__(self, *args, **kw):
        super(TestValidationState, self).__init__(*args, **kw)

    def test_counter_is_incremented(self):
        cherrypy.root = self.Controller()
        # parameter values are irrelevant
        url = '/validation?a=1&b=2&c.a=3&c.b=4'
        testutil.create_request(url)
        body = cherrypy.response.body[0]
        msg = "Validation state is not handled properly"
        # 4 == 1 (a) + 1(b) + 1(c.a) + 1(c.b)
        self.failUnless('counter: 4' in body, msg)



# Formatting configuration for locale gu

languages={'gu': u'\u0a97\u0ac1\u0a9c\u0ab0\u0abe\u0aa4\u0ac0'}

countries={'IN': u'\u0aad\u0abe\u0ab0\u0aa4'}

months=[u'\u0a9c\u0abe\u0aa8\u0acd\u0aaf\u0ac1\u0a86\u0ab0\u0ac0', u'\u0aab\u0ac7\u0aac\u0acd\u0ab0\u0ac1\u0a86\u0ab0\u0ac0', u'\u0aae\u0abe\u0ab0\u0acd\u0a9a', u'\u0a8f\u0aaa\u0acd\u0ab0\u0abf\u0ab2', u'\u0aae\u0ac7', u'\u0a9c\u0ac2\u0aa8', u'\u0a9c\u0ac1\u0ab2\u0abe\u0a88', u'\u0a91\u0a97\u0ab8\u0acd\u0a9f', u'\u0ab8\u0aaa\u0acd\u0a9f\u0ac7\u0aae\u0acd\u0aac\u0ab0', u'\u0a91\u0a95\u0acd\u0a9f\u0acd\u0aac\u0ab0', u'\u0aa8\u0ab5\u0ac7\u0aae\u0acd\u0aac\u0ab0', u'\u0aa1\u0abf\u0ab8\u0ac7\u0aae\u0acd\u0aac\u0ab0']

abbrMonths=[u'\u0a9c\u0abe\u0aa8\u0acd\u0aaf\u0ac1', u'\u0aab\u0ac7\u0aac\u0acd\u0ab0\u0ac1', u'\u0aae\u0abe\u0ab0\u0acd\u0a9a', u'\u0a8f\u0aaa\u0acd\u0ab0\u0abf\u0ab2', u'\u0aae\u0ac7', u'\u0a9c\u0ac2\u0aa8', u'\u0a9c\u0ac1\u0ab2\u0abe\u0a88', u'\u0a91\u0a97\u0ab8\u0acd\u0a9f', u'\u0ab8\u0aaa\u0acd\u0a9f\u0ac7', u'\u0a91\u0a95\u0acd\u0a9f\u0acb', u'\u0aa8\u0ab5\u0ac7', u'\u0aa1\u0abf\u0ab8\u0ac7']

days=[u'\u0ab8\u0acb\u0aae\u0ab5\u0abe\u0ab0', u'\u0aae\u0a82\u0a97\u0ab3\u0ab5\u0abe\u0ab0', u'\u0aac\u0ac1\u0aa7\u0ab5\u0abe\u0ab0', u'\u0a97\u0ac1\u0ab0\u0ac1\u0ab5\u0abe\u0ab0', u'\u0ab6\u0ac1\u0a95\u0acd\u0ab0\u0ab5\u0abe\u0ab0', u'\u0ab6\u0aa8\u0abf\u0ab5\u0abe\u0ab0', u'\u0ab0\u0ab5\u0abf\u0ab5\u0abe\u0ab0']

abbrDays=[u'\u0ab8\u0acb\u0aae', u'\u0aae\u0a82\u0a97\u0ab3', u'\u0aac\u0ac1\u0aa7', u'\u0a97\u0ac1\u0ab0\u0ac1', u'\u0ab6\u0ac1\u0a95\u0acd\u0ab0', u'\u0ab6\u0aa8\u0abf', u'\u0ab0\u0ab5\u0abf']

dateFormats={'medium': '%Y %%(abbrmonthname)s %d ', 'full': '%%(dayname)s, %Y %%(monthname)s %d ', 'long': '%Y %%(monthname)s %d ', 'short': '%y/%m/%d'}

numericSymbols={'group': ',', 'nativeZeroDigit': u'\u0ae6', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': '.', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}



"""Graphical user interface for i18n administration"""

import os
import sys
import time
import shutil
import codecs

import pygettext
import msgfmt
import catalog

import cherrypy
from cherrypy.lib.cptools import serveFile

import turbogears
from turbogears import controllers, expose, i18n


class Internationalization(controllers.RootController):
    """I18N administration tool.

    Collect your strings, add and manage locales,
    edit and compile your catalogs.

    """

    __label__ = "admi18n"
    __version__ = "0.1"
    __author__ = "Ronald Jaramillo"
    __email__ = "ronald@checkandshare.com"
    __copyright__ = "Copyright 2005 Ronald Jaramillo"
    __license__ = "MIT"

    baseTemplate = 'turbogears.toolbox.admi18n'
    languages = None
    icon = "/tg_static/images/admi18n.png"
    need_project = True

    def __init__(self, currentProject=None):
        self.currentProject = currentProject
        if not self.currentProject:
            self.currentProject = os.getcwd()

    def get_languages(self):
        if not self.languages:
            self.languages = i18n.get_languages()

        return self.languages

    def remove_locale(self, code):
        locales = self.locales_directory()
        if type(code) != type([]):
            code = [code]

        for c in code:
            path = os.path.join(locales, c)
            try:
                shutil.rmtree(path)
            except OSError,e:
                print e
                return

    def compile_catalogs(self, codes):
        locales = self.locales_directory()
        for code in codes.split(','):
            path = os.path.join(locales, code, 'LC_MESSAGES', 'messages.po')
            if not os.path.exists(path):
                continue

            dest = path.replace('.po', '.mo')
            #run msgfmt on file...
            msgfmt.make(path, dest)

    def merge_catalogs(self, codes):
        locales = self.locales_directory()
        src = os.path.join(locales, 'messages.pot')
        paths = []
        for code in codes.split(','):
            path = os.path.join(locales, code, 'LC_MESSAGES', 'messages.po')
            if not os.path.exists(path):
                continue

            paths.append(path)
        catalog.merge(src, paths)

    def add_locale(self, code):
        locales = self.locales_directory()
        path = os.path.join(locales, code)
        try:
            os.mkdir(path)
        except OSError, e:
            print e
            return

        path = os.path.join(path, 'LC_MESSAGES')
        try:
            os.mkdir(path)
        except OSError, e:
            print e
            return

        src = os.path.join(locales, 'messages.pot')
        dest = os.path.join(path, 'messages.po')
        shutil.copy(src, dest)

    [expose(format='json')]
    def po_upload(self, myFile, code):
        path = os.path.join(self.locales_directory(),
                code, 'LC_MESSAGES', 'messages.po')
        f = codecs.open(path, 'wb', 'utf-8')
        f.write(unicode(myFile.file.read(), 'utf-8', errors='replace'))
        f.close()
        raise cherrypy.HTTPRedirect(turbogears.url('language', code=code))

    def google_translate(self, code, from_lang, to_lang, args):
        path = os.path.join(self.locales_directory(),
            code, 'LC_MESSAGES', 'messages.po')
        for arg in args:
            if not 'text_' in arg:
                continue
            msg_id = args[arg].strip()
            if '\r' in msg_id:
                msg_id = '\n'.join(msg_id.splitlines())
            translated = i18n.utils.google_translate(
                    from_lang, to_lang, msg_id)
            catalog.update(path, msg_id, translated)

    [expose(format='json')]
    def update_catalog(self, code, msg_id, msg_text):
        path = os.path.join(self.locales_directory(),
                code, 'LC_MESSAGES', 'messages.po')
        catalog.update(path, msg_id, msg_text)
        return 'ok'

    [expose(template='%s.po_view' % baseTemplate)]
    def po_view(self, code, sort_by=None, dir=None,
            from_lang=None, to_lang=None, **kargs):
        visible_checkbox = False
        if from_lang and to_lang:
            visible_checkbox = True
            self.google_translate(code, from_lang, to_lang, kargs)
        path = os.path.join(self.locales_directory(),
                code, 'LC_MESSAGES', 'messages.po')
        return dict(code=code, catalog=catalog.items(path, sort_by, dir),
                visible_checkbox=visible_checkbox)

    [expose(template='%s.language' % baseTemplate)]
    def language(self, code):
        path = os.path.join(self.locales_directory(),
            code, 'LC_MESSAGES', 'messages.po')
        po_message_file = dict(path=path,
            modified=time.ctime(os.path.getmtime(path)),
            size=os.path.getsize(path))
        return dict(code=code,
            language=self.language_for_code(code),
            po_message_file=po_message_file)

    [expose(format='json')]
    def language_list(self):
        return dict(languages=self.get_languages())

    [expose(template='%s.languageManagement' % baseTemplate)]
    def language_management(self, add=None, rem=None, compile=None, merge=None):
        if add:
            self.add_locale(add)
        if rem:
            self.remove_locale(rem)
        if compile:
            self.compile_catalogs(compile)
        if merge:
            self.merge_catalogs(merge)
        return dict(languages=list(),
            locales=self.project_locales())

    def language_for_code(self, code):
        for c, language in self.get_languages():
            if c == code:
                return language

    def locales_directory(self):
        locales_dir = os.path.join(self.currentProject, 'locales')
        if not os.path.isdir(locales_dir):
            try:
                os.mkdir(locales_dir)
            except OSError, e:
                print e
                return
        return locales_dir

    def project_locales(self):
        locales = []
        locales_dir = self.locales_directory()
        if not locales_dir:
            return locales

        for item in os.listdir(locales_dir):
            path = os.path.join(locales_dir, item)
            if not os.path.isdir(path):
                continue

            language = self.language_for_code(item)
            if not language:
                continue

            modified = '-'
            compiled = '-'
            po = os.path.join(path, 'LC_MESSAGES', 'messages.po')
            mo = os.path.join(path, 'LC_MESSAGES', 'messages.mo')
            if os.path.exists(po):
                modified = time.ctime(os.path.getmtime(po))

            if os.path.exists(mo):
                compiled = time.ctime(os.path.getmtime(mo))

            locales.append(dict(
                             code=item,
                             language=language,
                             coverage=0,
                             status=0,
                             modified=modified,
                             compiled=compiled
                           ))
        return locales

    def project_files(self):
        p = turbogears.util.get_package_name()
        base_level = len([x for x in p.split(os.sep) if x])
        fl, dct, visibility = [], {}, {}

        def collect_files(file_list, dirpath, namelist):
            level = len([x for x in dirpath.split(os.sep) if x]) - base_level
            slot0 = dict( # directory info
                dir=os.path.dirname(dirpath),
                file_name=os.path.basename(dirpath),
                path=dirpath, isdir=True, level=level)
            slot1 = list() # children directories info
            slot2 = list() # children files info
            slots = (slot0, slot1, slot2)
            dct[dirpath] = slots
            if level:
                dct[os.path.dirname(dirpath)][1].append(slots)
            else:
                file_list.append(slots)

            namelist.sort()
            for name in namelist[:]:
                if name.startswith('.') or \
                        name in ['static', 'sqlobject-history']:
                    namelist.remove(name)
                    continue
                p = os.path.join(dirpath, name)
                if os.path.isfile(p) and \
                        os.path.splitext(name)[-1] in ['.py','.kid','.tmpl']:
                    slot2.append(dict(dir=dirpath, file_name=name,
                        path=p, isdir=os.path.isdir(p), level=level+1))
            # decide if current directory (and ancestors) should be visible
            visibility[dirpath] = bool(slot2)
            if slot2:
                while not visibility.get(os.path.dirname(dirpath), True):
                    dirpath = os.path.dirname(dirpath)
                    visibility[dirpath] = True

        os.path.walk(p, collect_files, fl)
        return [x for x in turbogears.util.flatten_sequence(fl)
                if not x["isdir"] or visibility[x["path"]]]

    def collect_string_for_files(self, files):
        if not files:
            return

        params = ['', '-v']
        for file in files:
            params.append(file)

        pygettext.sys.argv = params
        pygettext.main()

        pot = os.path.join(self.currentProject, 'messages.pot')
        if not os.path.exists(pot):
            return

        locales = self.locales_directory()
        filename = os.path.join(locales, 'messages.pot')
        try:
            os.rename(pot, filename)
        except OSError, e:
            print e

    def pot_message_file(self):
        locales = self.locales_directory()
        pot = os.path.join(locales, 'messages.pot')
        if not os.path.exists(pot):
            return

        return dict(name='messages.pot', path=pot,
            modified=time.ctime(os.path.getmtime(pot)),
            size=os.path.getsize(pot))

    [expose()]
    def lang_file(self, code):
        # serve static file, the code can be pot or a lang code
        locales = self.locales_directory()
        if code == 'pot':
            path = os.path.join(locales, 'messages.pot')
        else:
            path = os.path.join(locales, code)
            path = os.path.join(path, 'LC_MESSAGES')
            path = os.path.join(path, 'messages.po')
        if 'If-Modified-Since' in cherrypy.request.headers:
            del cherrypy.request.headers['If-Modified-Since'] # see ticket #879
        return serveFile(path, "application/x-download", "attachment")

    [expose(template='%s.stringCollection' % baseTemplate)]
    def string_collection(self, files=[]):
        if files:
            if type(files) != type([]):
                files = [files]
            self.collect_string_for_files(files)
        return dict(project_files=self.project_files(),
            pot_message_file=self.pot_message_file())

    [expose(template='%s.internationalization' % baseTemplate)]
    def index(self):
        return dict()




from turbogears import view
from turbogears.widgets.datagrid import DataGrid


def setup_module():
    if not view.engines:
        view.load_engines()


class Foo:

    def __init__(self, name, subtype, text):
        self.name = name
        self.subtype = subtype
        self.text = text

    def _get_excerpt(self):
        return self.text + '...'
    excerpt = property(_get_excerpt)

class User:

    def __init__(self, ID, name, emailAddress):
        self.userId = ID
        self.name = name
        self.emailAddress = emailAddress
    displayName = property(lambda self: self.name.capitalize())


class TestDataGrid:

    def test_column_declaration(self):
        """Test standard column declaration styles."""
        grid = DataGrid(name='grid', fields=[
            DataGrid.Column('name', options=dict(foobar=123)),
            ('Subtype', 'subtype'),
            DataGrid.Column('text', 'excerpt', 'TEXT')])
        d = dict(value='value')
        grid.update_params(d)
        get_field = d['get_field']
        # 'headers' and 'collist' only supported for backward compatibility
        assert d['headers'] == ['Name', 'Subtype', 'TEXT']
        assert d['collist'] == ['name', 'column-1', 'text']
        assert d['columns'][0].get_option('foobar') == 123
        assert grid.get_column('name').options['foobar'] == 123
        row = Foo('spa1', 'fact', 'thetext')
        assert 'spa1' == get_field(row, 'name')
        assert 'fact' == get_field(row, 'column-1')
        assert 'thetext...' == get_field(row, 'text')

    def test_alternative_column_declaration(self):
        """Test alternative column declaration styles."""
        g = lambda: None
        grid = DataGrid(fields=[
            (g), ('t', g), ('t', g, dict(p=0)),
            DataGrid.Column('n', g, 't', dict(p=0))])
        for i in range(4):
            name = i == 3 and 'n' or 'column-%d' % i
            c = grid[name]
            assert c.name == name
            title = i == 0 and name.capitalize() or 't'
            assert c.title == title
            assert c.getter == g
            options = i > 1 and dict(p=0) or dict()
            assert c.options == options

    def test_empty_column_title(self):
        """Test automatic and empty column titles."""
        grid = DataGrid(fields=[
            (None, 'a'), ('', 'a'), ('title', 'a')])
        assert grid['column-0'].title == 'Column-0'
        assert grid['column-1'].title == ''
        assert grid['column-2'].title == 'title'

    def test_column_alignment(self):
        """Test controlling column alignment."""
        grid = DataGrid(fields=[
            ('left col', lambda d: d[0]),
            ('center col', lambda d: d[1], dict(align='center')),
            ('right col', lambda d: d[2], dict(align='right'))])
        output = grid.render([('left data', 'center data', 'right data')])
        assert '<td>left data</td><td align="center">center data</td>' \
            '<td align="right">right data</td>' in output

    def test_template_overridal(self):
        """Test overriding the template."""
        grid = DataGrid(fields=[
                ('Name', 'name'),
                ('Subtype', 'subtype'),
                ('TEXT', Foo._get_excerpt)],
            template = "turbogears.fastdata.templates.datagrid")
        d = dict(value='value')
        grid.update_params(d)

    def test_wiki_samples(self):
        """Test that sample code on DataGridWidget wiki page actually works."""
        grid = DataGrid(fields=[
            ('ID', 'userId'),
            ('Name', 'displayName'),
            ('E-mail', 'emailAddress')])
        users = [User(1, 'john', 'john@foo.net'),
            User(2, 'fred', 'fred@foo.net')]
        output = grid.render(users)
        assert '<td>2</td><td>Fred</td><td>fred@foo.net</td>' in output
        grid = DataGrid(fields=[
            ('Name', lambda row: row[1]),
            ('Country', lambda row: row[2]),
            ('Age', lambda row: row[0])])
        data = [(33, "Anton Bykov", "Bulgaria"),
            (23, "Joe Doe", "Great Britain"),
            (44, "Pablo Martelli", "Brazil")]
        output = grid.render(data)
        assert '<td>Joe Doe</td><td>Great Britain</td><td>23</td>' in output




import os.path

from turbogears import config
from turbogears.i18n import sogettext

def setup_module():
    basedir = os.path.join(os.path.dirname(os.path.dirname(
            os.path.dirname(os.path.abspath(__file__)))), "tests")
    locale_dir = os.path.join(basedir, 'locale')
    config.update({
        'i18n.locale_dir': locale_dir,
        'i18n.domain': 'messages',
        'i18n.default_locale': 'en',
        'i18n.get_locale': lambda: 'en',
        'i18n.run_template_filter': False,
        'sqlobject.dburi': "sqlite:///:memory:"
    })
    sogettext.create_so_catalog(["en", "fi"], "messages")



import re
import turbogears
import cherrypy

import turbogears.testutil as testutil
from turbogears import widgets, validators

def test_required_fields():
    """
    Required field are automatically discovered from the form validator and marked
    with the "requiredfield" css class.
    """
    class MyFields(widgets.WidgetsList):
        name = widgets.TextField(validator=validators.String())
        comment = widgets.TextArea(validator=validators.String(not_empty=True))
    form = widgets.TableForm(fields=MyFields())

    class MyRoot(turbogears.controllers.RootController):
        def test(self):
            return dict(form=form)
        test = turbogears.expose(template=".form")(test)

    cherrypy.root = MyRoot()
    testutil.create_request("/test")
    output = cherrypy.response.body[0].lower()

    print output
    name_p = 'name="comment"'
    class_p = 'class="textarea requiredfield"'
    assert (re.compile('.*'.join([class_p, name_p])).search(output) or
            re.compile('.*'.join([name_p, class_p])).search(output)
    )
    name_p = 'name="name"'
    class_p = 'class="textfield"'
    assert (re.compile('.*'.join([class_p, name_p])).search(output) or
            re.compile('.*'.join([name_p, class_p])).search(output)
    )

def test_two_form_in_the_same_page():
    """
    Two different forms containing some fields with the same name can be
    validated and re-displayed on the same page with right values and errors.
    """
    class MyFields(widgets.WidgetsList):
        age = widgets.TextField(validator=validators.Int())
        email = widgets.TextArea(validator=validators.Email())
    form1 = widgets.TableForm(name="form1", fields=MyFields())
    form2 = widgets.TableForm(name="form2", fields=MyFields())

    class MyRoot(turbogears.controllers.RootController):
        def test(self):
            return dict(form1=form1, form2=form2)
        test = turbogears.expose(template=".two_forms")(test)
        test = turbogears.validate(form=form1)(test)
        test = turbogears.error_handler(test)(test)

        def test2(self):
            return dict(form=form2)
        test2 = turbogears.expose(template=".form")(test2)
        test2 = turbogears.validate(form=form2)(test2)
        test2 = turbogears.error_handler(test2)(test2)

        def test3(self):
            return dict(form=form1)
        test3 = turbogears.expose(template=".form")(test3)
        test3 = turbogears.validate(form=form2)(test3)
        test3 = turbogears.error_handler(test3)(test3)

    cherrypy.root = MyRoot()
    testutil.create_request("/test?age=foo&email=bar")
    output = cherrypy.response.body[0].lower()
    iv = cherrypy.request.input_values
    validation_errors = cherrypy.request.validation_errors
    validated_form = cherrypy.request.validated_form

    print output
    print iv
    print validation_errors
    print validated_form
    assert form1 is validated_form
    assert form1.is_validated
    assert form2 is not validated_form
    assert not form2.is_validated
    value_p = 'value="foo"'
    id_p = 'id="form1_age"'
    assert (re.compile('.*'.join([value_p, id_p])).search(output) or
            re.compile('.*'.join([id_p, value_p])).search(output)
    )
    value_p = 'value="foo"'
    id_p = 'id="form2_age"'
    assert not (re.compile('.*'.join([value_p, id_p])).search(output) or
            re.compile('.*'.join([id_p, value_p])).search(output)
    )

    cherrypy.root = MyRoot()
    testutil.create_request("/test2?age=foo&email=bar")
    output = cherrypy.response.body[0].lower()
    iv = cherrypy.request.input_values
    validation_errors = cherrypy.request.validation_errors
    validated_form = cherrypy.request.validated_form

    print output
    print iv
    print validation_errors
    print validated_form
    assert form1 is not validated_form
    assert not form1.is_validated
    assert form2 is validated_form
    assert form2.is_validated
    assert 'value="foo"' in output
    assert '>bar<' in output
    assert 'class="fielderror"' in output

    cherrypy.root = MyRoot()
    testutil.create_request("/test3?age=foo&email=bar")
    output = cherrypy.response.body[0].lower()
    iv = cherrypy.request.input_values
    validation_errors = cherrypy.request.validation_errors
    validated_form = cherrypy.request.validated_form

    print output
    print iv
    print validation_errors
    print validated_form
    assert form1 is not validated_form
    assert not form1.is_validated
    assert form2 is validated_form
    assert form2.is_validated
    assert 'value="foo"' not in output
    assert '>bar<' not in output
    assert 'class="fielderror"' not in output

def test_repeating_fields():
    repetitions = 5

    class MyFields(widgets.WidgetsList):
        name = widgets.TextField(validator=validators.String(not_empty=True))
        comment = widgets.TextField(validator=validators.String())

    form = widgets.TableForm(name="form", fields=[widgets.RepeatingFieldSet(
            name="repeat", fields=MyFields(), repetitions=repetitions)])

    class MyRoot(turbogears.controllers.RootController):
        def test(self):
            return dict(form=form)
        test = turbogears.expose(template=".form")(test)

        def test_value(self):
            value = dict(repeat=[{'name':'foo', 'comment':'hello'},
                                 None,
                                 None,
                                 {'name':'bar', 'comment':'byebye'}])
            return dict(form=form, value=value)
        test_value = turbogears.expose(template=".form")(test_value)

        def test_validation(self):
            return dict(form=form)
        test_validation = turbogears.expose(template=".form")(test_validation)
        test_validation = turbogears.validate(form=form)(test_validation)
        test_validation = turbogears.error_handler(test_validation)(test_validation)

    cherrypy.root = MyRoot()
    testutil.create_request("/test")
    output = cherrypy.response.body[0].lower()
    for i in range(repetitions):
        assert 'id="form_repeat_%i"' % i in output
        assert 'name="repeat-%i.name"' % i in output
        assert 'id="form_repeat_%i_name"' % i in output
        assert 'name="repeat-%i.comment"' % i in output
        assert 'id="form_repeat_%i_comment"' % i in output

    cherrypy.root = MyRoot()
    testutil.create_request("/test_value")
    output = cherrypy.response.body[0].lower()
    name_p = 'name="repeat-0.name"'
    value_p = 'value="foo"'
    assert (re.compile('.*'.join([value_p, name_p])).search(output) or
            re.compile('.*'.join([name_p, value_p])).search(output)
    )
    name_p = 'name="repeat-0.comment"'
    value_p = 'value="hello"'
    assert (re.compile('.*'.join([value_p, name_p])).search(output) or
            re.compile('.*'.join([name_p, value_p])).search(output)
    )
    name_p = 'name="repeat-3.name"'
    value_p = 'value="bar"'
    assert (re.compile('.*'.join([value_p, name_p])).search(output) or
            re.compile('.*'.join([name_p, value_p])).search(output)
    )
    name_p = 'name="repeat-3.comment"'
    value_p = 'value="byebye"'
    assert (re.compile('.*'.join([value_p, name_p])).search(output) or
            re.compile('.*'.join([name_p, value_p])).search(output)
    )
    name_p = 'name="repeat-1.name"'
    value_p = 'value=""'
    assert (re.compile('.*'.join([value_p, name_p])).search(output) or
            re.compile('.*'.join([name_p, value_p])).search(output)
    )
    name_p = 'name="repeat-1.comment"'
    value_p = 'value=""'
    assert (re.compile('.*'.join([value_p, name_p])).search(output) or
            re.compile('.*'.join([name_p, value_p])).search(output)
    )

    cherrypy.root = MyRoot()
    testutil.create_request("/test_validation?repeat-0.name=foo&repeat-1.name="
        "&repeat-2.name=bar&repeat-3.name=&repeat-4.name=")
    output = cherrypy.response.body[0].lower()
    validation_errors = cherrypy.request.validation_errors
    assert validation_errors.has_key("repeat")
    assert isinstance(validation_errors["repeat"], list)
    assert validation_errors["repeat"][0] is None
    assert validation_errors["repeat"][1].has_key("name")
    assert validation_errors["repeat"][2] is None
    assert validation_errors["repeat"][3].has_key("name")
    assert validation_errors["repeat"][4].has_key("name")



import dispatch
from turbogears import config
from turbogears.util import get_model
try:
    from sqlalchemy import MetaData, exceptions, Table, String, Unicode
    from turbogears.database import metadata, get_engine
except ImportError: # if not available, complain only at run-time
    get_engine = None
else:
    try:
        from sqlalchemy import Text, UnicodeText
    except ImportError: # SQLAlchemy < 0.4.3
        Text, UnicodeText = String, Unicode

[dispatch.generic()]
def sacommand(command, args):
    pass

[sacommand.around("command and command != 'help' and not get_engine")]
def no_engine(command, args):
    print "Error: SQLAlchemy not installed."

[sacommand.when("command == 'help'")]
def help(command, args):
    print """TurboGears SQLAlchemy Helper

tg-admin sql command [options]

Available commands:
    create  Create tables
    execute Execute SQL statements
    help    Show help
    list    List tables that appear in the model
    status  Show differences between model and database
"""

[sacommand.when("command == 'create'")]
def create(command, args):
    print "Creating tables at %s" % (config.get("sqlalchemy.dburi"))
    get_engine()
    get_model()
    metadata.create_all()

[sacommand.when("command == 'list'")]
def list_(command, args):
    get_model()
    for tbl in metadata.tables.values():
        print tbl.fullname

[sacommand.when("command == 'execute'")]
def execute(command, args):
    eng = get_engine()
    for cmd in args[2:]:
        ret = eng.execute(cmd)
        try:
            print list(ret)
        except:
            # Proceed silently if the command produced no results
            pass

[sacommand.when("command == 'status'")]
def status(command, args):
    get_engine()
    get_model()
    ret = compare_metadata(metadata, MetaData(metadata.bind))
    for l in ret:
        print l
    if not ret:
        print "Database matches model"

def indent(ls):
    return ['    ' + l for l in ls]

def compare_metadata(pym, dbm):
    rc = []
    for pyt in pym.tables.values():
        try:
            dbt = Table(pyt.name, dbm, autoload=True, schema=pyt.schema)
        except exceptions.NoSuchTableError:
            rc.extend(("Create table " + pyt.fullname, ''))
        else:
            ret = compare_table(pyt, dbt)
            if ret:
                rc.append("Change table " + pyt.fullname)
                rc.extend(indent(ret) + [''])
    return rc

def compare_table(pyt, dbt):
    rc = []
    dbcols = dict([(s.lower(), s) for s in dbt.columns.keys()])
    any_pkey = bool([1 for c in dbt.columns if c.primary_key])
    for pyc in pyt.columns:
        name = pyc.name.lower()
        if dbcols.has_key(name):
            ret = compare_column(pyc, dbt.columns[dbcols[name]], any_pkey)
            if ret:
                rc.append("Change column " + pyc.name)
                rc.extend(indent(ret))
            dbcols.pop(name)
        else:
            rc.append("Add column " + pyc.name)
    for dbcol in dbcols:
        rc.append("Remove column " + dbcol)
    return rc

def compare_column(pyc, dbc, any_pkey):
    rc = []
    pyt, dbt = pyc.type, dbc.type

    # Table reflection cannot recognize Unicode, so check only for String
    if isinstance(pyt, Unicode):
        pyt = String(pyt.length)
    elif isinstance(pyt, UnicodeText):
        pyt = Text(pyt.length)

    # Check type
    if not isinstance(dbt, pyt.__class__):
        rc.append('Change type to ' + pyt.__class__.__name__)

    # Check length (for strings)
    else:
        if isinstance(pyt, String):
            if pyt.length != dbt.length:
                rc.append('Change length to ' + str(pyt.length))

    # Check primary key
    if any_pkey and dbc.primary_key != pyc.primary_key:
        rc.append(pyc.primary_key and 'Make primary key' or 'Remove primary key')

    # TODO: Check foreign keys

    # Check default
    if (dbc.default is not None and pyc.default is not None
            and dbc.default != pyc.default):
        rc.append('Change default to ' + str(pyc.default.arg))

    # Check index
    if dbc.index is not None and dbc.index != pyc.index:
        rc.append(pyc.index and 'Add index' or 'Remove index')

    return rc



# Formatting configuration for locale kw

languages={'kw': 'kernewek'}

countries={'GB': 'Rywvaneth Unys'}

months=['Mys Genver', 'Mys Whevrel', 'Mys Merth', 'Mys Ebrel', 'Mys Me', 'Mys Efan', 'Mys Gortheren', 'Mye Est', 'Mys Gwyngala', 'Mys Hedra', 'Mys Du', 'Mys Kevardhu']

abbrMonths=['Gen', 'Whe', 'Mer', 'Ebr', 'Me', 'Efn', 'Gor', 'Est', 'Gwn', 'Hed', 'Du', 'Kev']

days=['De Lun', 'De Merth', 'De Merher', 'De Yow', 'De Gwener', 'De Sadorn', 'De Sul']

abbrDays=['Lun', 'Mth', 'Mhr', 'Yow', 'Gwe', 'Sad', 'Sul']

dateFormats={'medium': '%Y %%(abbrmonthname)s %d ', 'full': '%%(dayname)s, %Y %%(monthname)s %d ', 'long': '%Y %%(monthname)s %d ', 'short': '%y/%m/%d'}

numericSymbols={'group': ',', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': '.', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}




import types
from math import ceil
import logging
import warnings

try:
    set
except NameError: # Python 2.3
    from sets import Set as set

import cherrypy
try:
    import sqlobject
except ImportError:
    sqlobject = None

try:
    import sqlalchemy
except ImportError:
    sqlalchemy = None

import turbogears
from turbogears.controllers import redirect
from turbogears.decorator import weak_signature_decorator
from turbogears.view import variable_providers
from formencode.variabledecode import variable_encode
from turbogears.widgets import PaginateDataGrid
from turbogears.util import add_tg_args

log = logging.getLogger("turbogears.paginate")


# lists of databases that lack support for OFFSET
# this will need to be updated periodically as modules change
_so_no_offset = 'mssql maxdb sybase'.split()
_sa_no_offset = 'mssql maxdb access'.split()

# this is a global that is set the first time paginate() is called
_simulate_offset = None

# these are helper classes for getting data that has no table column
class attrwrapper:
    """Helper class for accessing objec attributes."""
    def __init__(self, name):
        self.name = name
    def __call__(self, obj):
        for name in self.name.split('.'):
            obj = getattr(obj, name)
        return obj

class itemwrapper:
    """Helper class for dicitionary access."""
    def __init__(self, name):
        self.name = name
    def __call__(self, obj):
        return obj[self.name]


def paginate(var_name, default_order='', default_reversed=None, limit=10,
            max_limit=0, allow_limit_override=None, max_pages=5,
            max_sort=1000, dynamic_limit=None):
    """The famous TurboGears paginate decorator.

    @param var_name: The variable name that the paginate decorator will try
    to control. This key must be present in the dictionnary returned from your
    controller in order for the paginate decorator to be able to handle it.
    @type var_name: string

    @param default_order: The column name(s) that will be used to orde
    pagination results. Due to the way pagination is implemented specifying a
    default_order will override any result ordering performed in the controller.
    @type default_order: string or a list of strings. Any string starting with
    "-" (minus sign) indicates a reverse order for that field/column.

    @param default_reversed: Deprecated, use default_order with minus sign.
    @type default_reversed: Boolean

    @param limit: The hard-coded limit that the paginate decorator will impose
    on the number of "var_name" to display at the same time. This value can be
    overridden by the use of the dynamic_limit keyword argument.
    @type limit: integer

    @param max_limit: The maximum number to which the imposed limit can be
    increased using the "var_name"_tgp_limit keyword argument in the URL.
    If this is set to 0, no dynamic change at all will be allowed;
    if it is set to None, any change will be allowed.
    @type max_limit: int

    @param allow_limit_override: Deprecated, use max_limit.
    @type allow_limit_override: Boolean

    @param max_pages: Used to generate the tg.paginate.pages variable. If the
    page count is larger than max_pages, tg.paginate.pages will only contain
    the page numbers surrounding the current page at a distance of max_pages/2.
    A zero value means that all pages will be shown, no matter how much.
    @type max_pages: integer

    @param max_sort: The maximum number of records that will be sorted in
    memory if the data cannot be sorted using SQL. If set to 0, sorting in
    memory will never be performed; if set to None, no limit will be imposed.
    @type max_sort: integer

    @param dynamic_limit: If specified, this parameter must be the name
    of a key present in the dictionary returned by your decorated
    controller. The value found for this key will be used as the limit
    for our pagination and will override the other settings, the hard-coded
    one declared in the decorator itself AND the URL parameter one.
    This enables the programmer to store a limit settings inside the
    application preferences and then let the user manage it.
    @type dynamic_limit: string

    """

    if default_reversed is not None:
        warnings.warn("default_reversed is deprecated."
            " Use default_order='-field' to indicate"
            " default reversed order, or"
            " default_order=['field1', '-field2, 'field3']"
            " for multiple fields.", DeprecationWarning, 2)
    if allow_limit_override is not None:
        warnings.warn("allow_limit_override is deprecated."
            " Use max_limit to specify an upper bound for limit.",
            DeprecationWarning, 2)

    def entangle(func):

        get = turbogears.config.get

        def decorated(func, *args, **kw):

            def kwpop(name, default=None):
                return kw.pop(var_name + '_tgp_' + name,
                    kw.pop('tg_paginate_' + name, default))

            page = kwpop('no')
            if page is None:
                page = 1
            elif page == 'last':
                page = None
            else:
                try:
                    page = int(page)
                    if page < 1:
                        raise ValueError
                except (TypeError, ValueError):
                    page = 1
                    if get('paginate.redirect_on_out_of_range'):
                        cherrypy.request.params[var_name + '_tgp_no'] = page
                        redirect(cherrypy.request.path_info, cherrypy.request.params)

            try:
                limit_ = int(kwpop('limit'))
                if max_limit is not None and not (
                        allow_limit_override and max_limit == 0):
                    if max_limit <= 0:
                        raise ValueError
                    limit_ = min(limit_, max_limit)
            except (TypeError, ValueError):
                limit_ = limit
            order = kwpop('order')
            ordering = kwpop('ordering')

            log.debug("paginate params: page=%s, limit=%s, order=%s",
                page, limit_, order)

            # get the output from the decorated function
            output = func(*args, **kw)
            if not isinstance(output, dict):
                return output

            try:
                var_data = output[var_name]
            except KeyError:
                raise KeyError("paginate: var_name"
                    " (%s) not found in output dict" % var_name)
            if not hasattr(var_data, '__getitem__') and callable(var_data):
                # e.g. SQLAlchemy query class
                var_data = var_data()
                if not hasattr(var_data, '__getitem__'):
                    raise TypeError('Paginate variable is not a sequence')

            if dynamic_limit:
                try:
                    dyn_limit = output[dynamic_limit]
                except KeyError:
                    raise KeyError("paginate: dynamic_limit"
                        " (%s) not found in output dict" % dynamic_limit)
                limit_ = dyn_limit

            if ordering:
                ordering = str(ordering).split(',')
            else:
                ordering = default_order or []
                if isinstance(ordering, basestring):
                    # adapt old style default_order to new style
                    if default_reversed:
                        ordering = "-" + ordering
                    ordering = [ordering]
                elif default_reversed:
                    raise ValueError("paginate: default_reversed (deprecated)"
                        " only allowed when default_order is a basestring")

            if order:
                order = str(order)
                log.debug('paginate: ordering was %s, sort is %s',
                    ordering, order)
                sort_ordering(ordering, order)
            log.debug('paginate: ordering is %s', ordering)

            try:
                row_count = len(var_data)
            except TypeError:
                try: # SQL query
                    row_count = var_data.count() or 0
                except AttributeError: # other iterator
                    var_data = list(var_data)
                    row_count = len(var_data)

            if ordering:
                var_data = sort_data(var_data, ordering,
                    max_sort is None or 0 < row_count <= max_sort)

            # If limit is zero then return all our rows
            if not limit_:
                limit_ = row_count or 1

            page_count = int(ceil(float(row_count)/limit_))

            if page is None:
                page = max(page_count, 1)
                if get('paginate.redirect_on_last_page'):
                    cherrypy.request.params[var_name + '_tgp_no'] = page
                    redirect(cherrypy.request.path_info, cherrypy.request.params)
            elif page > page_count:
                page = max(page_count, 1)
                if get('paginate.redirect_on_out_of_range'):
                    cherrypy.request.params[var_name + '_tgp_no'] = page
                    redirect(cherrypy.request.path_info, cherrypy.request.params)

            offset = (page-1) * limit_

            pages_to_show = _select_pages_to_show(page, page_count, max_pages)

            # remove pagination parameters from request
            input_values =  variable_encode(cherrypy.request.params.copy())
            input_values.pop('self', None)
            for input_key in input_values.keys():
                if (input_key.startswith(var_name + '_tgp_') or
                        input_key.startswith('tg_paginate_')):
                    del input_values[input_key]

            paginate_instance = Paginate(
                current_page=page,
                limit=limit_,
                pages=pages_to_show,
                page_count=page_count,
                input_values=input_values,
                order=order,
                ordering=ordering,
                row_count=row_count,
                var_name=var_name)

            cherrypy.request.paginate = paginate_instance
            if not hasattr(cherrypy.request, 'paginates'):
                cherrypy.request.paginates = dict()
            cherrypy.request.paginates[var_name] = paginate_instance

            # we replace the var with the sliced one
            endpoint = offset + limit_
            log.debug("paginate: slicing data between %d and %d",
                offset, endpoint)

            global _simulate_offset
            if _simulate_offset is None:
                _simulate_offset = get('paginate.simulate_offset', None)
                if _simulate_offset is None:
                    _simulate_offset = False
                    so_db = get('sqlobject.dburi', 'NOMATCH:').split(':', 1)[0]
                    sa_db = get('sqlalchemy.dburi', 'NOMATCH:').split(':', 1)[0]
                    if so_db in _so_no_offset or sa_db in _sa_no_offset:
                        _simulate_offset = True
                        log.warning("paginate: simulating OFFSET,"
                            " paginate may be slow"
                            " (disable with paginate.simulate_offset=False)")

            if _simulate_offset:
                var_data = iter(var_data[:endpoint])
                # skip over the number of records specified by offset
                for i in xrange(offset):
                    var_data.next()
                # return the records that remain
                output[var_name] = list(var_data)
            else:
                try:
                    output[var_name] = var_data[offset:endpoint]
                except TypeError:
                    for i in xrange(offset):
                        var_data.next()
                    output[var_name] = [var_data.next()
                        for i in xrange(offset, endpoint)]

            return output

        if not get('tg.strict_parameters', False):
            # add hint that paginate parameters shall be left intact
            args = set()
            for arg in 'no', 'limit', 'order', 'ordering':
                args.add(var_name + '_tgp_' + arg)
                args.add('tg_paginate_' + arg)
            add_tg_args(func, args)
        return decorated

    return weak_signature_decorator(entangle)


def _paginate_var_provider(d):
    """Auxiliary function for providing the paginate variable."""
    paginate = getattr(cherrypy.request, 'paginate', None)
    if paginate:
        d.update(dict(paginate=paginate))
    paginates = getattr(cherrypy.request, 'paginates', None)
    if paginates:
        d.update(dict(paginates=paginates))
variable_providers.append(_paginate_var_provider)


class Paginate:
    """Class for paginate variable provider."""

    def __init__(self, current_page, pages, page_count, input_values,
                 limit, order, ordering, row_count, var_name):

        self.var_name = var_name
        self.pages = pages
        self.limit = limit
        self.page_count = page_count
        self.current_page = current_page
        self.input_values = input_values
        self.order = order
        self.ordering = ordering
        self.row_count = row_count
        self.first_item = page_count and ((current_page - 1) * limit + 1) or 0
        self.last_item = min(current_page * limit, row_count)

        self.reversed = ordering and ordering[0][0] == '-'

        # If ordering is empty, don't add it.
        input_values = {var_name + '_tgp_limit': limit}
        if ordering:
            input_values[var_name + '_tgp_ordering'] = ','.join(ordering)
        self.input_values.update(input_values)

        if current_page < page_count:
            self.input_values.update({
                var_name + '_tgp_no': current_page + 1,
                var_name + '_tgp_limit': limit
            })
            self.href_next = turbogears.url(
                cherrypy.request.path_info, self.input_values)
            self.input_values.update({
                var_name + '_tgp_no': 'last',
                var_name + '_tgp_limit': limit
            })
            self.href_last = turbogears.url(
                cherrypy.request.path_info, self.input_values)
        else:
            self.href_next = None
            self.href_last = None

        if current_page > 1:
            self.input_values.update({
                var_name + '_tgp_no': current_page - 1,
                var_name + '_tgp_limit': limit
            })
            self.href_prev = turbogears.url(
                cherrypy.request.path_info, self.input_values)
            self.input_values.update({
                var_name + '_tgp_no': 1,
                var_name + '_tgp_limit': limit
            })
            self.href_first = turbogears.url(
                cherrypy.request.path_info, self.input_values)
        else:
            self.href_prev = None
            self.href_first = None

    def get_href(self, page, order=None, reverse_order=None):
        # Note that reverse_order is not used.  It should be cleaned up here
        # and in the template.  I'm not removing it now because I don't want
        # to break the API.
        order = order or None
        input_values = self.input_values.copy()
        input_values[self.var_name + '_tgp_no'] = page
        if order:
            input_values[ self.var_name + '_tgp_order'] = order
        return turbogears.url('', input_values)


def _select_pages_to_show(current_page, page_count, max_pages=None):
    """Auxiliary function for getting the range of pages to show."""
    if max_pages is not None and max_pages > 0:
        start = current_page - (max_pages // 2) - (max_pages % 2) + 1
        end = start + max_pages - 1
        if start < 1:
            start, end = 1, min(page_count, max_pages)
        elif end > page_count:
            start, end = max(1, page_count - max_pages + 1), page_count
    else:
        start, end = 1, page_count
    return xrange(start, end + 1)


# Auxiliary functions for dealing with columns and SQL

def sort_ordering(ordering, sort_name):
    """Rearrange ordering based on sort_name."""
    try:
        index = ordering.index(sort_name)
    except ValueError:
        try:
            index = ordering.index('-' + sort_name)
        except ValueError:
            ordering.insert(0, sort_name)
        else:
            del ordering[index]
            ordering.insert(0, (index and '-' or '') + sort_name)
    else:
        del ordering[index]
        ordering.insert(0, (not index and '-' or '') + sort_name)

def sqlalchemy_get_column(colname, var_data):
    """Return a column from sqlalchemy var_data based on colname."""
    try:
        mapper = var_data.mapper
    except AttributeError: # SQLAlchemy >= 0.5
        try:
            mapper = var_data._mapper_zero()
        except AttributeError: # SQLAlchemy <= 0.4 and SelectResullts
            mapper = var_data._query.mapper
    propnames = colname.split('.')
    colname = propnames.pop()
    for propname in propnames:
        prop = mapper.get_property(
            propname, resolve_synonyms=True, raiseerr=False)
        if not prop:
            break
        mapper = prop.mapper
    if mapper.column_prefix:
        # for SQLAlchemy <= 0.4.0 we may need to remove the column_prefix
        return getattr(mapper.c, colname,
            getattr(mapper.c, colname[len(mapper.column_prefix):], None))
    else:
        return getattr(mapper.c, colname, None)

def sqlobject_get_column(colname, var_data):
    """Return a column from sqlobject var_data based on colname."""
    return getattr(var_data.sourceClass.q, colname, None)

def sql_get_column(colname, var_data):
    """Return a column from var_data based on colname."""
    if sqlalchemy:
        try:
            return sqlalchemy_get_column(colname, var_data)
        except AttributeError:
            pass
    if sqlobject:
        try:
            return sqlobject_get_column(colname, var_data)
        except AttributeError:
            pass
    raise TypeError('Cannot find columns of paginate variable')

def sqlalchemy_order_col(col, descending=False):
    """Return an sqlalchemy ordered column for col."""
    if descending:
        return sqlalchemy.sql.desc(col)
    else:
        return sqlalchemy.sql.asc(col)

def sqlobject_order_col(col, descending=False):
    """Return an sqlobject ordered column for col."""
    if descending:
        return sqlobject.DESC(col)
    else:
        return col

def sql_order_col(col, ascending=True):
    """Return an ordered column for col."""
    if sqlalchemy and isinstance(col, sqlalchemy.sql.ColumnElement):
        return sqlalchemy_order_col(col, ascending)
    elif sqlobject and isinstance(col, types.InstanceType):
        # Sadly, there is no better way to check for sqlobject col type
        return sqlobject_order_col(col, ascending)
    raise TypeError("Expected Column, but got %s" % type(col))

def sort_data(data, ordering, in_memory=True):
    """Sort data based on ordering.

    Tries to sort the data using SQL whenever possible,
    otherwise sorts the data as list in memory unless in_memory is false.

    """
    try:
        order_by = data.order_by # SQLAlchemy
        get_column, order_col = sqlalchemy_get_column, sqlalchemy_order_col
    except AttributeError:
        try:
            order_by = data.orderBy # SQLObject
            get_column, order_col = sqlobject_get_column, sqlobject_order_col
        except AttributeError:
            order_by = None
    order_cols = []
    key_cols = []
    num_ascending = num_descending = 0
    for order in ordering:
        if order[0] == '-':
            order = order[1:]
            descending = True
        else:
            descending = False
        if order_by:
            col = get_column(order, data)
            if col is not None:
                order_cols.append(order_col(col, descending))
                continue
        if not order_cols:
            key_cols.append((order, descending))
            if descending:
                num_descending += 1
            else:
                num_ascending += 1
    if order_by and order_cols:
        data = order_by(order_cols)
    if key_cols:
        if in_memory:
            data = list(data)
            if not data:
                return data
            wrapper = isinstance(data[0], dict) and itemwrapper or attrwrapper
            keys = [(wrapper(col[0]), col[1]) for col in key_cols]
            if num_ascending == 0 or num_descending == 0:
                reverse = num_ascending == 0
                keys = [key[0] for key in keys]
                if len(key_cols) == 1:
                    key = keys[0]
                else:
                    key = lambda row: [key(row) for key in keys]
            else:
                reverse = num_descending > num_ascending
                def reverse_key(key, descending):
                    if reverse == descending:
                        return key
                    else:
                        keys = map(key, data)
                        try:
                            keys = list(set(keys))
                        except TypeError: # unhashable
                            keys.sort()
                            return lambda row: -keys.index(key(row))
                        else:
                            keys.sort()
                            keys = dict([(k, -n) for n, k in enumerate(keys)])
                            return lambda row: keys[key(row)]
                keys = [reverse_key(*key) for key in keys]
                key = lambda row: [key(row) for key in keys]
            try:
                data.sort(key=key, reverse=reverse)
            except TypeError: # Python 2.3
                if reverse:
                    cmpkey = lambda row1, row2: cmp(key(row2), key(row1))
                else:
                    cmpkey = lambda row1, row2: cmp(key(row1), key(row2))
                data.sort(cmpkey)
        else:
            log.debug("paginate: sorting in memory not allowed")
    return data



# Formatting configuration for locale af

languages={'af': 'Afrikaanse'}

countries={'ZA': 'Suid-Afrika'}

months=['Januarie', 'Februarie', 'Maart', 'April', 'Mei', 'Junie', 'Julie', 'Augustus', 'September', 'Oktober', 'November', 'Desember']

abbrMonths=['Jan', 'Feb', 'Mar', 'Apr', 'Mei', 'Jun', 'Jul', 'Aug', 'Sep', 'Okt', 'Nov', 'Des']

days=['Maandag', 'Dinsdag', 'Woensdag', 'Donderdag', 'Vrydag', 'Saterdag', 'Sondag']

abbrDays=['Ma', 'Di', 'Wo', 'Do', 'Vr', 'Sa', 'So']

dateFormats={'medium': '%Y %%(abbrmonthname)s %d ', 'full': '%%(dayname)s, %Y %%(monthname)s %d ', 'long': '%Y %%(monthname)s %d ', 'short': '%y/%m/%d'}

numericSymbols={'group': ',', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': '.', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}



from sqlobject import *

__connection__ = connectionForURI("sqlite:///:memory:")
hub = __connection__

class Client(SQLObject):
    name = StringCol()
    physical_address=SingleJoin('Address')
    #shipping_address=SingleJoin('Address')

class Address(SQLObject):
    client=ForeignKey('Client',default=None)
    street=StringCol()

Client.createTable(ifNotExists=True)
Address.createTable(ifNotExists=True)



"""Generic widget to present and manipulate data in a grid (tabular) form.

See also: http://trac.turbogears.org/turbogears/wiki/DataGridWidget

"""

from turbogears.widgets import Widget, CSSLink, static
from turbogears.widgets.base import CoreWD

NoDefault = object()

__all__ = ["DataGrid", "PaginateDataGrid"]


class DataGrid(Widget):
    """Generic widget to present and manipulate data in a grid (tabular) form.

    The columns to build the grid from are specified with fields ctor argument
    which is a list. Currently an element can be either a two-element tuple or
    instance of DataGrid.Column class. If tuple is used it a Column is then
    build out of it, first element is assumed to be a title and second element -
    field accessor.

    You can specify columns' data statically, via fields ctor parameter, or
    dynamically, by via 'fields' key.

    """
    css = [CSSLink(static, "grid.css")]
    template = "turbogears.widgets.templates.datagrid"
    fields = None

    class Column:
        """Simple struct that describes single DataGrid column.

        Column has:
          - a name, which allows to uniquely identify column in a DataGrid
          - getter, which is used to extract field's value
          - title, which is displayed in the table's header
          - options, which is a way to carry arbitrary user-defined data

        """
        def __init__(self, name, getter=None, title=None, options=None):
            if not name:
                raise ValueError, 'name is required'
            if getter:
                if callable(getter):
                    self.getter = getter
                else: # assume it's an attribute name
                    self.getter = DataGrid.attrwrapper(getter)
            else:
                self.getter = DataGrid.attrwrapper(name)
            self.name = name
            self.title = title is None and name.capitalize() or title
            self.options = options or {}
        def get_option(self, name, default=NoDefault):
            if name in self.options:
                return self.options[name]
            if default is NoDefault: # no such key and no default is given
                raise KeyError(name)
            return default
        def get_field(self, row):
            return self.getter(row)
        def __str__(self):
            return "<DataGrid.Column %s>" % self.name

    class attrwrapper:
        """Helper class that returns an object's attribute when called.

        This allows to access 'dynamic' attributes (properties) as well as
        simple static ones, and also allows nested access.

        """
        def __init__(self, name):
            assert isinstance(name, str)
            self.name = name
        def __call__(self, obj):
            for name in self.name.split('.'):
                obj = getattr(obj, name)
            return obj

    def __init__(self, fields=None, **kw):
        super(DataGrid, self).__init__(**kw)
        if fields:
            self.fields = fields
        if self.fields is None:
            self.fields = []
        self.columns = self._parse(self.fields)

    def get_column(self, name):
        """Return DataGrid.Column with specified name.

        Raises KeyError if no such column exists.

        """
        for col in self.columns:
            if col.name == name:
                return col
        raise KeyError(name)

    def __getitem__(self, name):
        """Shortcut to get_column."""
        return self.get_column(name)

    def get_field_getter(columns):
        """Return a function to access the fields of table by row, col."""
        idx = {} # index columns by name
        for col in columns:
            idx[col.name] = col
        def _get_field(row, col):
            return idx[col].get_field(row)
        return _get_field
    get_field_getter = staticmethod(get_field_getter)

    def update_params(self, d):
        super(DataGrid, self).update_params(d)
        if d.get('fields'):
            fields = d.pop('fields')
            columns = self._parse(fields)
        else:
            columns = self.columns[:]
        d['columns'] = columns
        d['get_field'] = self.get_field_getter(columns)
        # this is for backward compatibility
        d['headers'] = [col.title for col in columns]
        d['collist'] = [col.name for col in columns]

    def _parse(self, fields):
        """Parse field specifications into a list of Columns.

        A specification can be a DataGrid.Column, an accessor
        (attribute name or function), a tuple (title, accessor)
        or a tuple (title, accessor, options).

        """
        columns = []
        names = {} # keep track of names to ensure there are no dups
        for n, col in enumerate(fields):
            if not isinstance(col, self.Column):
                if isinstance(col, str) or callable(col):
                    name_or_f = col
                    title = options = None
                else:
                    title, name_or_f = col[:2]
                    try:
                        options = col[2]
                    except IndexError:
                        options = None
                name = 'column-' + str(n)
                col = self.Column(name, name_or_f, title, options)
            if col.name in names:
                raise ValueError('Duplicate column name: %s' % name)
            columns.append(col)
            names[col.name] = 1
        return columns


class DataGridDesc(CoreWD):

    name = "DataGrid"

    for_widget = DataGrid(fields=[('Name', lambda row: row[1]),
                                  ('Country', lambda row: row[2]),
                                  ('Age', lambda row: row[0])],
                          default=[(33, "Anton Bykov", "Bulgaria"),
                                   (23, "Joe Doe", "Great Britain"),
                                   (44, "Pablo Martelli", "Brazil")])


class PaginateDataGrid(DataGrid):
    """A data grid widget that supports the paginate decorator."""

    template = "turbogears.widgets.templates.paginate_datagrid"


class PaginateDataGridDesc(CoreWD):

    name = "PaginateDataGrid"

    for_widget = DataGridDesc.for_widget

    class Paginate(object):
        # paginate var mock-up
        page_count = 5
        pages = range(1, page_count + 1)
        limit = 3
        current_page = page_count // 2
        href_first = "javascript:alert('This is only a mock-up.')"
        href_prev = href_next = href_last = href_first
        get_href = lambda self, page, **kw: self.href_first

    def display(self, *args, **kw):
        # activate paginate var provider
        import turbogears.paginate
        from cherrypy import request
        request.paginate = self.Paginate()
        return super(PaginateDataGridDesc, self).display(*args, **kw)



"""The TurboGears identity management package.

@TODO: Laundry list of things yet to be done:
    * IdentityFilter should support HTTP Digest Auth
    * Also want to support Atom authentication (similar to digest)

"""

import md5
import sha
import threading

import cherrypy
import pkg_resources
import logging
log = logging.getLogger('turbogears.identity')

import turbogears
from turbogears.util import request_available, load_class
from turbogears.identity.exceptions import *


def create_default_provider():
    """Create default identity provider.

    Creates an identity provider according to what is found in
    the configuration file for the current TurboGears application

    Returns an identity provider instance or
    raises an IdentityConfigurationException.

    """
    provider_plugin = turbogears.config.get('identity.provider', 'sqlobject')
    plugins = pkg_resources.iter_entry_points(
        'turbogears.identity.provider', provider_plugin)

    log.debug("Loading provider from plugin: %s", provider_plugin)

    provider_class = None
    for entrypoint in plugins:
        provider_class = entrypoint.load()
        break

    if not provider_class:
        provider_class = load_class(provider_plugin)

    if not provider_class:
        raise IdentityConfigurationException(
            "IdentityProvider plugin missing: %s" % provider_plugin )
    else:
        return provider_class()


class IdentityWrapper(object):
    """A wrapper class for the thread local data.

    This allows developers to access the current user information via
    turbogears.identity.current and get the identity for the current request.

    """

    def identity(self):
        try:
            identity = cherrypy.request.identity
        except AttributeError:
            identity = None
        if not identity:
            if not request_available():
                raise RequestRequiredException()
            raise IdentityManagementNotEnabledException()
        return identity

    def __getattr__(self, name):
        """Return the named attribute of the global state."""
        identity = self.identity()
        if name == '__str__':
            return identity.__str__
        elif name == '__repr__':
            return identity.__repr__
        else:
            return getattr(identity, name)

    def __setattr__(self, name, value):
        """Stash a value in the global state."""
        identity = self.identity()
        setattr(identity, name, value)


class ProviderWrapper(object):

    def __getattr__(self, name):
        try:
            provider = cherrypy.request.identityProvider
        except AttributeError:
            try:
                provider = create_default_provider()
            except Exception:
                provider = None

        if provider is None:
            if not request_available():
                raise RequestRequiredException()
            raise IdentityManagementNotEnabledException()

        return getattr(provider, name)


current = IdentityWrapper()
current_provider = ProviderWrapper()


def was_login_attempted():
    try:
        return cherrypy.request.identity_login_attempted
    except AttributeError:
        return False


def set_login_attempted( flag ):
    cherrypy.request.identity_login_attempted = flag


def set_current_identity(identity):
    cherrypy.request.identity = identity
    try:
        cherrypy.request.user_name = identity.user_name
    except AttributeError:
        cherrypy.request.user_name = None


def set_current_provider( provider ):
    cherrypy.request.identityProvider = provider


from turbogears.identity.conditions import *

def _encrypt_password(algorithm, password):
    """Hash the given password with the specified algorithm.

    Valid values for algorithm are 'md5' and 'sha1'.
    All other algorithm values will be essentially a no-op.

    """
    hashed_password = password
    # The algorithms don't work with unicode objects, so decode first.
    if isinstance(password, unicode):
        password_8bit = password.encode('utf-8')
    else:
        password_8bit = password
    if algorithm == 'md5':
        hashed_password =  md5.new(password_8bit).hexdigest()
    elif algorithm == 'sha1':
        hashed_password = sha.new(password_8bit).hexdigest()
    elif algorithm == 'custom':
        custom_encryption_path = turbogears.config.get(
            'identity.custom_encryption', None)
        if custom_encryption_path:
            custom_encryption = turbogears.util.load_class(
                custom_encryption_path)
        if custom_encryption:
            hashed_password = custom_encryption(password_8bit)
    # Make sure the hashed password is a unicode object at the end of the
    # process, because SQLAlchemy _wants_ that for Unicode columns.
    if not isinstance(hashed_password, unicode):
        hashed_password = hashed_password.decode('utf-8')
    return hashed_password

def encrypt_password(cleartext):
    return current_provider.encrypt_password(cleartext)


# declare what should be exported
__all__ = [ 'IdentityManagementNotEnabledException',
    'IdentityConfigurationException', 'IdentityFailure',
    'current', 'current_provider',
    'set_current_identity', 'set_current_provider',
    'set_identity_errors', 'get_identity_errors',
    'was_login_attempted', 'encrypt_password',
    'get_failure_url']



# -*- coding: utf-8 -*-

import sys
import os
import shutil
import tempfile
import unittest

from datetime import datetime

from turbogears import config
from turbogears.command.i18n import InternationalizationTool

tool = None

TEMPLATE = """
<html xmlns:py="http://purl.org/kid/ns#">
<head>
    <link py:strip="1" py:for="css in tg_css">${css.display()}</link>
    <link py:strip="1" py:for="js in tg_js_head">${js.display()}</link>
</head>
<body>
  <div>Some text to be i18n'ed</div>
  <div>This is text that has a kid-expression ${_('which is to be i18n')}</div>
  <div foo="${_('kid expression in attribute')}"/>
  <div foo="normal attribute text"/>
  <div lang="en">This is english, and it shouldn't be collected</div>
  <div lang="de">Dies ist Deutsch, und es sollte nicht aufgesammelt werden</div>
  <div>These are some entities that we shouldn't complain about: &nbsp;</div>
</body>
</html>
"""

class I18nToolTest(unittest.TestCase):
    """Test for the tg-admin i18n tool."""

    def setUp(self):
        global tool
        tool = InternationalizationTool('1.1')
        self.work_dir = tempfile.mkdtemp()
        self.src_dir = os.path.join(self.work_dir, 'src')
        self.locale_dir = os.path.join(self.work_dir, 'locale')
        os.mkdir(self.src_dir)
        config.update({
            'i18n.locale_dir': self.locale_dir,
            'i18n.domain': 'testmessages',
        })

    def tearDown(self):
        shutil.rmtree(self.work_dir)

    # XXX: these aren't really unit tests, rather functional tests.
    # Unfortunately the InternationalizationTool would need to be seriously
    # refactored to allow this.

    def test_collect_js_strings_utf8(self):
        """Strings from JS files with non-ascii chars (UTF-8) are collected correctly"""
        js_file = os.path.join(self.src_dir, "utf8.js")
        tf = open(js_file, "w")
        tf.write(u"msg = _('Schöne Grüße')".encode('utf-8'))
        tf.close()
        sys.argv = ['i18n.py', '--src-dir', self.src_dir, 'collect']
        tool.load_config = False
        tool.options, dummy = tool.parse_args()
        os.mkdir(self.locale_dir)
        tmp_handle, tmp_potfile = tempfile.mkstemp('.pot', 'tmp', self.locale_dir)
        tool.scan_js_files(tmp_potfile, [js_file])
        pot_content = os.read(tmp_handle, 999).decode('utf-8')
        os.close(tmp_handle)
        os.remove(tmp_potfile)
        os.rmdir(self.locale_dir)
        assert u'Schöne Grüße' in pot_content

    def test_collect_js_strings_latin1(self):
        """Strings from JS files with non-ascii chars (Latin-1) are collected correctly"""
        js_file = os.path.join(self.src_dir, "utf8.js")
        tf = open(js_file, "w")
        tf.write(u"msg = _('Schöne Grüße')".encode('latin1'))
        tf.close()
        sys.argv = ['i18n.py', '--src-dir', self.src_dir, '--js-encoding', 'latin1', 'collect']
        tool.load_config = False
        tool.options, dummy = tool.parse_args()
        os.mkdir(self.locale_dir)
        tmp_handle, tmp_potfile = tempfile.mkstemp('.pot', 'tmp', self.locale_dir)
        tool.scan_js_files(tmp_potfile, [js_file])
        pot_content = os.read(tmp_handle, 999).decode('utf-8')
        os.close(tmp_handle)
        os.remove(tmp_potfile)
        os.rmdir(self.locale_dir)
        assert u'Schöne Grüße' in pot_content

    def test_collect_template_strings(self):
        """Verify the locale directory got created as needed."""
        tf = open(os.path.join(self.src_dir, "test.kid"), "w")
        tf.write(TEMPLATE)
        tf.close()
        assert not os.path.isdir(self.locale_dir), "locale directory should not exist yet"
        sys.argv = ['i18n.py', '--src-dir', self.src_dir, 'collect']
        tool.load_config = False
        tool.run()
        assert os.path.isdir(self.locale_dir), "locale directory not created"
        tf = open(os.path.join(self.locale_dir, "testmessages.pot"))
        pot_content = tf.read()
        tf.close()
        assert "Some text to be i18n'ed" in pot_content
        assert "kid expression in attribute" in pot_content
        assert "normal attribute text" not in pot_content
        assert "it shouldn't be collected" not in pot_content
        assert "es sollte nicht aufgesammelt werden" not in pot_content



# Formatting configuration for locale vi_VN

languages={'ru': u'Ti\u1ebfng Nga', 'eo': u'Ti\u1ebfng Esp\xe9ranto', 'en': u'Ti\u1ebfng Anh', 'nl': u'Ti\u1ebfng H\xe0 Lan', 'ro': u'Ti\u1ebfng La M\xe3', 'pt': u'Ti\u1ebfng B\u1ed3 \u0110\xe0o Nha', 'hr': u'Ti\u1ebfng Cr\xf4atia', 'tr': u'Ti\u1ebfng Th\u1ed5 Nh\u0129 K\u1ef3', 'ca': u'Ti\u1ebfng Catalan', 'de': u'Ti\u1ebfng \u0110\u1ee9c', 'ko': u'Ti\u1ebfng H\xe0n Qu\u1ed1c', 'fr': u'Ti\u1ebfng Ph\xe1p', 'sv': u'Ti\u1ebfng Thu\u1ef5 \u0110i\u1ec3n', 'da': u'Ti\u1ebfng \u0110an M\u1ea1ch', 'vi': u'Ti\u1ebfng Vi\u1ec7t', 'it': u'Ti\u1ebfng \xdd', 'zh': u'Ti\u1ebfng Trung Qu\u1ed1c', 'hu': u'Ti\u1ebfng Hung-ga-ri', 'ja': u'Ti\u1ebfng Nh\u1eadt', 'es': u'Ti\u1ebfng T\xe2y Ban Nha'}

countries={'BD': u'B\u0103ng-la-\u0111\xe9t', 'BE': u'B\u1ec9', 'BF': u'Bu\u1ed1c-ki-na Pha-x\xf4', 'BG': 'Bun-ga-ri', 'BA': u'B\xf4-xni-a H\xe9c-x\xea-g\xf4-vi-na', 'BB': u'B\xe1c-ba-\u0111\u1ed1t', 'BN': u'Bru-n\xe2y', 'BO': u'B\xf4-li-vi-a', 'BH': 'Ba-ren', 'BI': u'Bu-run-\u0111i', 'BJ': u'B\xea-nanh', 'JM': 'Ha-mai-ca', 'BW': u'B\u1ed1t-xoa-na', 'WS': 'Xa-moa', 'BR': 'Bra-xin', 'BS': 'Ba-ha-ma', 'BY': u'B\xea-la-r\xfat', 'BZ': u'B\xea-li-x\xea', 'RU': 'Nga', 'RW': u'Ru-an-\u0111a', 'TM': u'Tu\u1ed1c-m\xea-ni-xtan', 'TJ': u'T\xe1t-gi-ki-xtan', 'RO': 'Ru-ma-ni', 'GW': u'Ghi-n\xea B\xedt-xao', 'GT': u'Goa-t\xea-ma-la', 'GR': u'Hy L\u1ea1p', 'GQ': u'Ghi-n\xea X\xedch-\u0111\u1ea1o', 'JP': u'Nh\u1eadt B\u1ea3n', 'GY': 'Guy-a-na', 'GE': 'Gru-di-a', 'GD': u'Gr\xea-na-\u0111a', 'GB': u'V\u01b0\u01a1ng qu\u1ed1c Anh', 'GA': u'Ga-b\xf4ng', 'SV': u'En-san-va-\u0111o', 'GN': u'Ghi-n\xea', 'GM': u'G\u0103m-bi-a', 'GH': 'Gha-na', 'OM': u'\xd4-man', 'TN': 'Tuy-ni-di', 'JO': u'Gi\xf3c-\u0111a-ni', 'SP': u'S\xe9cbia', 'HR': u'Cr\xf4-a-ti-a', 'HT': 'Ha-i-ti', 'HU': 'Hung-ga-ri', 'HN': u'H\xf4n-\u0111u-r\xe1t', 'PT': u'B\u1ed3 \xd0\xe0o Nha', 'PY': 'Pa-ra-goay', 'PA': 'Pa-na-ma', 'PG': u'Pa-pu-a Niu Ghi-n\xea', 'PE': u'P\xea-ru', 'PK': 'Pa-ki-xtan', 'PH': 'Phi-lip-pin', 'PL': 'Ba Lan', 'ZM': u'D\u0103m-bi-a', 'EH': u'Ph\xeda t\xe2y Sahara', 'EE': u'E-xt\xf4-ni-a', 'EG': u'Ai-c\u1eadp', 'ZA': 'Nam Phi', 'EC': u'\xca-cu-a-\u0111o', 'IT': u'\xdd', 'VN': u'Vi\u1ec7t Nam', 'SB': u'Qu\u1ea7n \u0111\u1ea3o X\xf4-l\xf4-m\xf4ng', 'ET': u'\xca-ti-\xf4-pi-a', 'SO': u'X\xf4-ma-li', 'ZW': u'Dim-ba-bu-\xea', 'ES': u'T\xe2y Ban Nha', 'ER': u'\xca-ri-t\u01a1-r\xea-a', 'MD': u'M\xf4n-\u0111\xf4-va', 'MG': u'Ma-\u0111a-g\xe1t-xca', 'MA': u'Ma-r\u1ed1c', 'MC': u'M\xf4-na-c\xf4', 'UZ': u'U-d\u01a1-b\xea-ki-xtan', 'MM': 'Mi-an-ma', 'ML': 'Ma-li', 'MN': u'M\xf4ng C\u1ed5', 'MH': u'Qu\u1ea7n \u0111\u1ea3o M\xe1c-san', 'MK': u'Ma-x\xea-\u0111\xf4-ni-a', 'MU': u'M\xf4-ri-x\u01a1', 'MT': 'Man-ta', 'MW': 'Ma-la-uy', 'MV': u'Man-\u0111i-v\u01a1', 'MR': u'M\xf4-ri-ta-ni', 'UG': u'U-gan-\u0111a', 'MY': 'Ma-lay-xi-a', 'MX': u'M\xea-hi-c\xf4', 'IL': 'I-xra-en', 'FR': u'Ph\xe1p', 'FI': u'Ph\u1ea7n Lan', 'FJ': 'Phi-gi', 'FM': u'Mi-cr\xf4-n\xea-xi-a', 'NI': 'Ni-ca-ra-goa', 'NL': u'H\xe0 Lan', 'NO': 'Na Uy', 'NA': 'Nam-mi-bi-a', 'VU': 'Va-nu-a-tu', 'NE': u'Ni-gi\xea', 'NG': u'Ni-gi\xea-ri-a', 'NZ': u'Niu Di-l\xe2n', 'NP': u'N\xea-pan', 'CI': u'B\u1edd Bi\u1ec3n Ng\xe0', 'CH': u'Thu\u1ef5 S\u0129', 'CO': u'C\xf4-l\xf4m-bi-a', 'CN': u'Trung Qu\u1ed1c', 'CM': u'Ca-m\u01a1-run', 'CL': u'Chi-l\xea', 'CA': u'Ca-na-\u0111a', 'CG': u'C\xf4ng-g\xf4', 'CF': u'C\u1ed9ng h\xf2a Trung Phi', 'CZ': u'C\u1ed9ng h\xf2a S\xe9c', 'CY': u'S\xedp', 'CR': u'C\u1ed1t-xta Ri-ca', 'CV': u'C\xe1p-ve', 'CU': 'Cu-ba', 'SZ': 'Xoa-di-len', 'SY': 'Xi-ri', 'KG': u'C\u01b0-r\u01a1-g\u01b0-xtan', 'KE': u'K\xea-ni-a', 'SR': 'Xu-ri-nam', 'KI': 'Ki-ri-ba-ti', 'KH': 'Cam-pu-chia', 'KN': u'Xan-k\xedt v\xe0 N\xea-vi', 'KM': u'C\xf4-m\xf4', 'ST': u'Xao T\xf4-m\xea v\xe0 Prin-xi-p\xea', 'SK': u'Xl\xf4-va-ki-a', 'KR': u'H\xe0n Qu\u1ed1c', 'SI': u'Xl\xf4-ven-ni-a', 'KP': u'Tri\u1ec1u Ti\xean', 'KW': u'C\xf4-o\xe9t', 'SN': u'X\xea-n\xea-gan', 'SM': u'Xan Ma-ri-n\xf4', 'SL': u'Xi-\xea-ra L\xea-\xf4n', 'SC': u'X\xe2y-sen', 'KZ': u'Ka-d\u1eafc-xtan', 'SA': u'\u1ea2-r\u1eadp X\xea-\xfat', 'SG': 'Xin-ga-po', 'SE': u'Thu\u1ef5 \u0110i\u1ec3', 'SD': u'Xu-\u0111\u0103ng', 'DJ': 'Gi-bu-ti', 'DK': u'\u0110an M\u1ea1c', 'DE': u'\xd0\u1ee9c', 'YE': u'Y-\xea-men', 'DZ': u'An-gi\xea-ri', 'US': u'H\u1ee3p ch\xfang qu\u1ed1c Hoa k\u1ef3', 'UY': 'U-ru-goay', 'LB': u'Li-b\u0103ng', 'LC': 'Xan Lu-xi', 'LA': u'L\xe0o', 'TV': 'Tu-va-lu', 'TW': u'\xd0\xe0i Loan', 'TT': u'Tri-ni-\u0111\xe1t v\xe0 T\xf4-ba-g\xf4', 'TR': u'Th\u1ed5 Nh\u0129 K\u1ef3', 'LK': 'Xri Lan-ca', 'LI': u'Lich-ten-xt\xean', 'LV': u'L\xe1t-vi-a', 'TO': u'T\xf4ng-ga', 'LT': 'Li-tu-a-ni-a', 'LU': u'L\xfac-x\u0103m-bua', 'LR': u'Li-b\xea-ri-a', 'LS': u'L\xea-x\xf4-th\xf4', 'TH': u'Th\xe1i Lan', 'TG': u'T\xf4-g\xf4', 'TD': u'S\xe1t', 'LY': 'Li-bi', 'VA': u'Va-ti-c\u0103ng', 'VC': u'Xan Vin-xen v\xe0 Gr\xea-na-din', 'AE': u'C\xe1c Ti\u1ec3u V\u01b0\u01a1ng qu\u1ed1c A-r\u1eadp Th\u1ed1ng nh\u1ea5t', 'VE': u'V\xea-n\xea-zu-\xea-la', 'AG': u'An-ti-gu-a v\xe0 Ba-bu-\u0111a', 'AF': u'\xc1p-ga-ni-xtan', 'IQ': u'I-r\u1eafc', 'IS': u'Ai-x\u01a1-len', 'IR': 'I-ran', 'AM': u'\xc1c-m\xea-ni-a', 'AL': 'An-ba-ni', 'AO': u'\u0102ng-g\xf4-la', 'AR': u'\xc1c-hen-ti-na', 'AU': u'\xdac \xd0\u1ea1i L\u1ee3i', 'AT': u'\xc1o', 'IN': u'\u1ea4n \xd0\u1ed9', 'TZ': 'Tan-da-ni-a', 'AZ': u'Ai-d\xe9c-bai-gian', 'IE': 'Ai-len', 'ID': u'Nam D\u01b0\u01a1ng', 'UA': 'U-crai-na', 'QA': 'Ca-ta', 'MZ': u'M\xf4-d\u0103m-b\xedch'}

months=[u'Th\xe1ng m\u1ed9t', u'Th\xe1ng hai', u'Th\xe1ng ba', u'Th\xe1ng t\u01b0', u'Th\xe1ng n\u0103m', u'Th\xe1ng s\xe1u', u'Th\xe1ng b\u1ea3y', u'Th\xe1ng t\xe1m', u'Th\xe1ng ch\xedn', u'Th\xe1ng m\u01b0\u1eddi', u'Th\xe1ng m\u01b0\u1eddi m\u1ed9t', u'Th\xe1ng m\u01b0\u1eddi hai']

abbrMonths=['Thg 1', 'Thg 2', 'Thg 3', 'Thg 4', 'Thg 5', 'Thg 6', 'Thg 7', 'Thg 8', 'Thg 9', 'Thg 10', 'Thg 11', 'Thg 12']

days=[u'Th\u1ee9 hai', u'Th\u1ee9 ba', u'Th\u1ee9 t\u01b0', u'Th\u1ee9 n\u0103m', u'Th\u1ee9 s\xe1u', u'Th\u1ee9 b\u1ea3y', u'Ch\u1ee7 nh\u1eadt']

abbrDays=['Th 2', 'Th 3', 'Th 4', 'Th 5', 'Th 6', 'Th 7', 'CN']

dateFormats={'medium': '%d-%m-%Y', 'full': '%%(dayname)s %d %%(monthname)s %Y', 'long': '%d %%(monthname)s %Y', 'short': '%d/%m/%Y'}

numericSymbols={'group': '.', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': ',', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}



# Formatting configuration for locale kl_GL

languages={'kl': 'kalaallisut'}

countries={'GL': 'Kalaallit Nunaat'}

months=['januari', 'februari', 'martsi', 'aprili', 'maji', 'juni', 'juli', 'augustusi', 'septemberi', 'oktoberi', 'novemberi', 'decemberi']

abbrMonths=['jan', 'feb', 'mar', 'apr', 'maj', 'jun', 'jul', 'aug', 'sep', 'okt', 'nov', 'dec']

days=['ataasinngorneq', 'marlunngorneq', 'pingasunngorneq', 'sisamanngorneq', 'tallimanngorneq', 'arfininngorneq', 'sabaat']

abbrDays=['ata', 'mar', 'pin', 'sis', 'tal', 'arf', 'sab']

dateFormats={'medium': '%%(abbrmonthname)s %d,%y', 'full': '%%(dayname)s %d %%(monthname)s %Y', 'long': '%d %%(monthname)s %Y', 'short': '%d/%m/%y'}

numericSymbols={'group': '.', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': ',', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}



from datetime import datetime

from sqlobject import SQLObject, SQLObjectNotFound, StringCol, DateTimeCol
from sqlobject.sqlbuilder import Update

from turbogears import config
from turbogears.database import PackageHub
from turbogears.util import load_class
from turbogears.visit.api import BaseVisitManager, Visit

hub = PackageHub("turbogears.visit")
__connection__ = hub

import logging

log = logging.getLogger("turbogears.visit.sovisit")

visit_class = None


class SqlObjectVisitManager(BaseVisitManager):

    def __init__(self, timeout):
        global visit_class
        visit_class_path = config.get("visit.soprovider.model",
            "turbogears.visit.sovisit.TG_Visit")
        visit_class = load_class(visit_class_path)
        if visit_class:
            log.info("Successfully loaded \"%s\"" % visit_class_path)
        # base-class' __init__ triggers self.create_model, so mappers need to
        # be initialized before.
        super(SqlObjectVisitManager, self).__init__(timeout)

    def create_model(self):
        try:
            # Create the Visit table if it doesn't already exist
            hub.begin()
            visit_class.createTable(ifNotExists=True)
            hub.commit()
            hub.end()
        except KeyError:
            # No database configured...
            log.info("No database is configured: Visit Tracking is disabled.")
            return

    def new_visit_with_key(self, visit_key):
        hub.begin()
        visit = visit_class(visit_key=visit_key,
            expiry=datetime.now()+self.timeout)
        hub.commit()
        hub.end()
        return Visit(visit_key, True)

    def visit_for_key(self, visit_key):
        """Return the visit for this key.

        Returns None if the visit doesn't exist or has expired.

        """
        try:
            expiry = self.queue[visit_key]
        except KeyError:
            visit = visit_class.lookup_visit(visit_key)
            if not visit:
                return None
            expiry = visit.expiry
        now = datetime.now()
        if expiry < now:
            return None
        # Visit hasn't expired, extend it
        self.update_visit(visit_key, now + self.timeout)
        return Visit(visit_key, False)

    def update_queued_visits(self, queue):
        if hub is None: # if VisitManager extension wasn't shutted down cleanly
            return
        hub.begin()
        try:
            conn = hub.getConnection()
            try:
                # Now update each of the visits with the most recent expiry
                for visit_key, expiry in queue.items():
                    u = Update(visit_class.q,
                        {visit_class.q.expiry.fieldName: expiry},
                        where=(visit_class.q.visit_key==visit_key))
                    conn.query(conn.sqlrepr(u))
                hub.commit()
            except:
                hub.rollback()
                raise
        finally:
            hub.end()


class TG_Visit(SQLObject):

    class sqlmeta:
        table = "tg_visit"

    visit_key = StringCol(length=40, alternateID=True,
            alternateMethodName="by_visit_key")
    created = DateTimeCol(default=datetime.now)
    expiry = DateTimeCol()

    def lookup_visit(cls, visit_key):
        try:
            return cls.by_visit_key(visit_key)
        except SQLObjectNotFound:
            return None
    lookup_visit = classmethod(lookup_visit)



"""Convenient validators and converters for data coming in from the web.

This module also imports everything from formencode.validators, so all
common validation routines are available here."""

import time
import re
from datetime import datetime
import warnings

import simplejson

from formencode import ForEach
from formencode import validators # to disambiguate the Number validator...

from formencode.validators import *
from formencode.compound import *
from formencode.api import Invalid, NoDefault
from formencode.schema import Schema

from turbojson import jsonify

from turbogears import util
from turbogears.i18n import format


def _(s): return s # dummy

# FormEncode should call TG's gettext function with domain = "FormEncode"
Validator.gettextargs['domain'] = 'FormEncode'


# Validators handling country names and/or languages

try:
    from formencode import national
except ImportError: # formencode version < 1.1
    national = util.Bunch(USStateProvince=StateProvince,
        USPhoneNumber=PhoneNumber,
        USPostalCode=PostalCode)
    try:
        national.update(dict(
            InternationalPhoneNumber=IPhoneNumberValidator))
    except NameError: # formencode version < 0.9
        pass
else:
    StateProvince = national.USStateProvince
    PhoneNumber = national.USPhoneNumber
    PostalCode = national.USPostalCode
    IPhoneNumberValidator = national.InternationalPhoneNumber


class TgFancyValidator(FancyValidator):
    gettextargs = {'domain': 'TurboGears'}


class Money(TgFancyValidator):
    """Validate a monetary value with currency."""

    messages = {
        'badFormat': _('Invalid number format'),
        'empty': _('Empty values not allowed'),
    }

    def __init__(self, allow_empty=None, *args, **kw):
        if allow_empty is not None:
            warnings.warn("Use not_empty instead of allow_empty",
                          DeprecationWarning, 2)
            not_empty = not allow_empty
            kw["not_empty"] = not_empty
        super(Money, self).__init__(*args, **kw)

    def _to_python(self, value, state):
        """Parse a string and return a float or integer."""
        try:
            return format.parse_decimal(value)
        except ValueError:
            raise Invalid(self.message('badFormat', state), value, state)

    def _from_python(self, value, state):
        """Return a string using the correct grouping."""
        return format.format_currency(value)


class Number(TgFancyValidator):
    """Validate a decimal number."""

    def _to_python(self, value, state):
        """Parse a string and return a float or integer."""
        if isinstance(value, basestring):
            try:
                value = format.parse_decimal(value)
            except ValueError:
                pass
        return validators.Number.to_python(value, state)

    def _from_python(self, value, state):
        """Return a string using the correct grouping."""
        dec_places = util.find_precision(value)
        if dec_places > 0:
            return format.format_decimal(value, dec_places)
        else:
            return format.format_number(value)


class DateTimeConverter(TgFancyValidator):
    """Convert between Python datetime objects and strings."""

    messages = {
        'badFormat': _('Invalid datetime format'),
        'empty': _('Empty values not allowed'),
    }

    def __init__(self, format="%Y/%m/%d %H:%M", allow_empty=None,
                *args, **kwargs):
        if allow_empty is not None:
            warnings.warn("Use not_empty instead of allow_empty",
                          DeprecationWarning, 2)
            not_empty = not allow_empty
            kwargs["not_empty"] = not_empty
        super(DateTimeConverter, self).__init__(*args, **kwargs)
        self.format = format

    def _to_python(self, value, state):
        """Parse a string and return a datetime object."""
        if value and isinstance(value, datetime):
            return value
        else:
            try:
                format = self.format
                if callable(format):
                    format = format()
                tpl = time.strptime(value, format)
            except ValueError:
                raise Invalid(self.message('badFormat', state), value, state)
            # shoudn't use time.mktime() because it can give OverflowError,
            # depending on the date (e.g. pre 1970) and underlying C library
            return datetime(year=tpl.tm_year, month=tpl.tm_mon, day=tpl.tm_mday,
                    hour=tpl.tm_hour, minute=tpl.tm_min, second=tpl.tm_sec)

    def _from_python(self, value, state):
        """Return a string representation of a datetime object."""
        if not value:
            return None
        elif isinstance(value, datetime):
            # Python stdlib can only handle dates with year greater than 1900
            format = self.format
            if callable(format):
                format = format()
            if format is None:
                format = "%Y-%m-%d"
            if value.year <= 1900:
                return strftime_before1900(value, format)
            else:
                return value.strftime(format)
        else:
            return value


# Improved FieldStorageUploadConverter heeding not_empty=False
# (see TurboGears ticket #1705, FormEncode bug #1905250)

class FieldStorageUploadConverter(TgFancyValidator):

    messages = {
        'notEmpty': _("Filename must not be empty"),
        }

    def _to_python(self, value, state=None):
        try:
            filename = value.filename
        except AttributeError:
            filename = None
        if not filename and self.not_empty:
            raise Invalid(self.message('notEmpty', state), value, state)
        return value


# For translated messages that are not wrapped in a Validator.messages
# dictionary, we need to reinstate the Turbogears gettext function under
# the name "_", with the "TurboGears" domain, so that the TurboGears.mo
# file is selected.
import turbogears.i18n
_ = lambda s: turbogears.i18n.gettext(s, domain='TurboGears')


class MultipleSelection(ForEach):
    """A default validator for SelectionFields with multiple selection."""

    if_missing = NoDefault
    if_empty = []

    def to_python(self, value, state=None):
        try:
            return super(MultipleSelection, self).to_python(value, state)
        except Invalid:
            raise Invalid(_("Please select at least a value"), value, state)


class Schema(Schema):
    """Modified Schema validator for TurboGears.

    A schema validates a dictionary of values, applying different validators
    (by key) to the different values.

    This modified Schema allows fields that do not appear in the fields
    parameter of your schema, but filters them out from the value dictionary.
    You might want to set filter_extra_fields to True when you're building a
    dynamic form with unpredictable keys and need these values.

    """

    filter_extra_fields = True
    allow_extra_fields = True
    if_key_missing = None

    def from_python(self, value, state=None):
        # The Schema shouldn't do any from_python conversion because
        # adjust_value already takes care of that for all childs.
        return value


class JSONValidator(TgFancyValidator):
    """A validator for JSON format."""

    def _from_python(self, value, state):
        return jsonify.encode(value)

    def _to_python(self, value, state):
        return simplejson.loads(value)


# Auxiliary functions

def _findall(text, substr):
    # Also finds overlaps
    sites = []
    i = 0
    while 1:
        j = text.find(substr, i)
        if j == -1:
            break
        sites.append(j)
        i = j+1
    return sites

_illegal_s = re.compile(r"((^|[^%])(%%)*%s)")

def strftime_before1900(dt, fmt):
    """strftime implementation supporting proleptic Gregorian dates before 1900.

    @see: http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/306860

    """
    if _illegal_s.search(fmt):
        raise TypeError("This strftime implementation does not handle %s")
    if dt.year > 1900:
        return dt.strftime(fmt)

    year = dt.year
    # For every non-leap year century, advance by
    # 6 years to get into the 28-year repeat cycle
    delta = 2000 - year
    off = 6*(delta // 100 + delta // 400)
    year += off

    # Move to around the year 2000
    year = year + ((2000 - year)//28)*28
    timetuple = dt.timetuple()
    s1 = time.strftime(fmt, (year,) + timetuple[1:])
    sites1 = _findall(s1, str(year))

    s2 = time.strftime(fmt, (year+28,) + timetuple[1:])
    sites2 = _findall(s2, str(year+28))

    sites = []
    for site in sites1:
        if site in sites2:
            sites.append(site)

    s = s1
    syear = "%4d" % (dt.year,)
    for site in sites:
        s = s[:site] + syear + s[site+4:]
    return s



# -*- coding: utf-8 -*-

from turbogears.toolbox.admi18n import pygettext

def test_support_explicit_lang():
    """the coma should not make the extractor bork
    """
    assert _('Something', 'en') == u'Something'
    assert _('New', 'en') == u'New'
